function PBTicTacToe_v10f(SaveAddress,NWCP1,itML,gen,contwin,MR,scorehis)
% Plasma Brain Playing Tic-Tac-Toe
% by Li Lin
% 11-18-2021
% Arguments:
% SaveAddress: The address to save results
% NWD: network depth
% NWCP1: Initial control parameters
% itML, gen, contwin, MR: The parameters from previous trainning
%
% Design of v2 and after:
% Input:
% NO   N2O    H2O
% N2   O3     He
% O2   H2     NO2
% Network Weight Control Parameters (NWCP): 27 effective in total
% 1) all ion reaction rate coefficients: 23 in total, 17 effective (due to the same m/z of + and - ions)
% 2) Te
% 3) 9 input gas Tg, other Tg are the mean value
% 
% Trainning in v4: use randomly generated board situation
% Evaluate each mutation using the strategy:
% Kevin Crowley, Robert S. Siegler (1993). 
% "Flexible Strategy Use in Young Children's Tic-Tac-Toe". 
% Cognitive Science. 17 (4): 531561. 
% doi:10.1016/0364-0213(93)90003-Q
%
% Update in v5: use temporally resolved control parameters
% Each parameter is also controlled by 10 temporal multipliers 
% 
% Update in v6: use multiple periods with multiple different peaks 
% of species temperatures to increase the depth of the network
%
% Update in v7: use the 10th OES peak between 1.77e10 and 1.8e10 as the
% output. No OES pickup time option. The pickup time is either fixed or the
% pickup is an average of OES. The 10th OES peak is OH(A2?+v0) => OH + h?(308.9nm)
% 
% Update in v8: use triangle wave for the OES-to-move transition
% in stead of using a 0-1 number representing the OES intensity in a range.
% This resolve the out-of-range problem. In v7, a panelty of score is used
% for the plasma mutation giving an out-of-range OES output.
%
% Update in v9: increased the initial ion densities using Marker_ion. Also,
% use sawtooth wave instead of the triangle wave for the one-OES peak output.
%
% Update in v10: use 9 independent plasma unit, each is same as the one
% used in v9, and each one-OES peak output represent one position on board
% 
% v10d: got 63% winning rate vs. Random
%
% Update in v10e: use ion temperature only, the code changing locates at
% the random mutation generator
% 
% Update in v10f: the issue of v10e is that even without training, the
% wining rate vs. random is still 60%. This is because without training,
% the plasma decision order is j = 1:3 in i = 1:3 due to all the plasma
% uints are same. But such a "j = 1:3 in i = 1:3" strategy vs random has a
% very high wining rate. The update is to use a random plasma unit
% distribution. The code changing locates at the function "OES2move"
% Also, a bug is fixed in the function "checkF"

disp('Plasma Brain Playing Tic-Tac-Toe 10-26-2021 by Li Lin');

% Data Information
ReactionFile = 'C:\Users\magic\Desktop\Simulation Database\Reactions01122021.txt';
BOLSIGAddress = 'C:\Users\magic\Desktop\Simulation Database\BOLSIG Database 2D\';
%ReactionFile = 'C:\Users\Admin\Desktop\Simulation Database\Reactions01122021.txt';
%BOLSIGAddress = 'C:\Users\Admin\Desktop\Simulation Database\BOLSIG Database 2D\';
%ReactionFile = 'C:\Users\cryst\Desktop\Simulation Database\Reactions01122021.txt';
%BOLSIGAddress = 'C:\Users\cryst\Desktop\Simulation Database\BOLSIG Database 2D\';

% Scientific Constants
kB = 1.38*10^(-23); % Boltzmann constant [J/K] 
Tgini = 300; % Gas bulk temperature [K]
P = 101325*0.01; % Gas bulk pressure [Pa]
nbulk = P/Tgini/kB/1000000; % Gas bulk density [cm-3]

% Load Database
disp('Loading Reactions...');
[SS,Ssize,Rsize,SIM,Reactants,SRN,...
    Marker_S,Marker_hv,Marker_hv_eff,~,~,Marker_ion] = LoadReactions(ReactionFile);
disp('Loading BOLSIG Data...');
BOLSIGData = importdata(strcat(BOLSIGAddress,'ReducedBOLSIGData.mat'));
BOLSIGData(:,3:size(BOLSIGData,2),:) = ...
    BOLSIGData(:,3:size(BOLSIGData,2),:)*10^6;

% Initialize Neural Network
disp('Preparing Neural Network...');
NWD = 9;
dt = 5*10^(-9); 
timerange = 50*10^(-9)*NWD/9;
PL = round(timerange/dt);
MRini = 0.2;
if NWCP1== 0
    MR = MRini; % mutation range
end

% Preparing Plasma
Sini = ones(Ssize,1)*10^12;
inputlist = {'NO','N2O','H2O','N2','O3','He','O2','H2','NO2'};
inputindex = zeros(size(inputlist,2),1);
q = 0;
for sn = 1:Ssize
    for hvsn = 1:size(Marker_hv)
        if Marker_hv(hvsn) == sn
            Sini(sn) = 10^6;
            break;
        end
    end
    for ionk = 1:size(Marker_ion)
        if Marker_ion(ionk,1)+Marker_ion(ionk,2) == sn
            Sini(sn) = nbulk/size(Marker_ion,1);
            if Marker_ion(ionk,1) ~= 0
                q = q+nbulk/size(Marker_ion,1);
            else
                q = q-nbulk/size(Marker_ion,1);
            end
        end
    end
    for i = 1:size(inputlist,2)
        if (strcmp(SS{sn},inputlist{i}) == 1)
            Sini(sn) = nbulk/9;
            inputindex(i) = sn;
        end
    end
end
Sini(Marker_S.e) = q;
Sini(Marker_S.M) = sum(Sini);

% Initialize NWCP
TIN = 10^4; % Training iteration number
NWC = {'H+','H-','O+','O-','OH+','OH-',...
    'NO+','NO-','NO2+','NO2-','O2+','O2-',...
    'e','H2O+','N2O+','He+','N2+','He2+',...
    'H2+','N+','O3-','N2O-','NO3-',...
    'NO','N2O','H2O','N2','O3','He','O2','H2','NO2'}; 
if NWCP1 == 0
    % the 23 ion temperature also controlled by pulse width (x ns in 100 ns) and the leading edge time (x ns in 100 ns)
    % first Ssize: the temperature (constant or high-level)
    % second Ssize: pulse width (x ns in 100 ns
    % third Ssize: leading edge time (x ns in 100 ns)
    NWCP1 = ones(Ssize*3*NWD,1);  
    for nwd = 1:NWD
        NWCP1((nwd-1)*3*Ssize+Ssize+1:(nwd-1)*3*Ssize+3*Ssize,1) = NWCP1((nwd-1)*3*Ssize+Ssize+1:(nwd-1)*3*Ssize+3*Ssize,1)*0.2;
    end
end
Marker_NWC = zeros(size(NWCP1,1),2);
for nwd = 1:NWD
    for sn = 1:Ssize
        for n = 1:size(NWC,2)
            if strcmp(SS{sn},NWC{n}) == 1
                Marker_NWC(sn+(nwd-1)*3*Ssize,1) = sn+(nwd-1)*3*Ssize;
                Marker_NWC(sn+Ssize+(nwd-1)*3*Ssize,1) = sn+Ssize+(nwd-1)*3*Ssize;
                Marker_NWC(sn+Ssize*2+(nwd-1)*3*Ssize,1) = sn+Ssize*2+(nwd-1)*3*Ssize;
                if n <= 23
                    tempstr = strsplit(SS{sn},'-');
                    if strcmp(tempstr{1,1},SS{sn}) ~= 1 % this is a negative ion
                        tempstr2 = strsplit(NWC{n-1},'+');
                        if strcmp(tempstr{1,1},tempstr2{1,1}) == 1 % there is a same species with positive sign before
                            for ssn = 1:Ssize
                                if strcmp(NWC{n-1},SS{ssn}) == 1
                                    Marker_NWC(sn+(nwd-1)*3*Ssize,2) = ssn+(nwd-1)*3*Ssize;
                                    Marker_NWC(ssn+(nwd-1)*3*Ssize,2) = sn+(nwd-1)*3*Ssize;
                                    Marker_NWC(sn+Ssize+(nwd-1)*3*Ssize,2) = ssn+Ssize+(nwd-1)*3*Ssize;
                                    Marker_NWC(ssn+Ssize+(nwd-1)*3*Ssize,2) = sn+Ssize+(nwd-1)*3*Ssize;
                                    Marker_NWC(sn+Ssize*2+(nwd-1)*3*Ssize,2) = ssn+Ssize*2+(nwd-1)*3*Ssize;
                                    Marker_NWC(ssn+Ssize*2+(nwd-1)*3*Ssize,2) = sn+Ssize*2+(nwd-1)*3*Ssize;
                                end
                            end
                        end
                    end
                    break;
                end
            end
        end % n in NWC loop end
    end % sn in Ssize loop end
end % NWD loop ennd

NWClist = zeros(size(NWC,2),1);
for n = 1:size(NWC,2)
    for sn = 1:Ssize
        if strcmp(SS{sn},NWC{n}) == 1
            NWClist(n) = sn;
        end
    end
end
%{
NWCP_TH = zeros(size(NWCP1,1),TIN);
if NWCP1 ~= 0
    for itn = 1:itML
        infile = strcat(SaveAddress,'PBTicTacToe Results---it',num2str(itn),'.mat');
        temp = importdata(infile);
        NWCP_TH(:,itn) = temp.NWCP(:,1);
    end
end
%}
% Initialize other parameters
BO = 2500; % board situation number
if NWCP1== 0
    itML = 0;
    contwin = 0;
    gen = 1;
end
playerN = 5;
totalmatchN = playerN*BO;
Shis = zeros(109,9,totalmatchN);
OEShisfull = zeros(36,9,totalmatchN);
boardhis = zeros(3,3,BO);
playerlist = zeros(BO,1);
if scorehis == 0
    scorehis = zeros(TIN,1);
end
Teini = 5; % [eV]

% Main Loop
figure('position',[400,300,1200,600]);
disp('Main Loop Start...');
while (itML < TIN)
    itML = itML+1; 
    PMrate = zeros(playerN,1);
    % make the mutation
    NWCP = ones(size(NWCP1,1),playerN);
    NWCP(:,1) = NWCP1(:,1);
    for pn = 2:playerN
        for i = 1:size(NWCP1,1)
            for nwd = 1:NWD
                if (i >= (nwd-1)*Ssize*3+1) && (i <= (nwd-1)*Ssize*3+Ssize) && (Marker_NWC(i,1) ~= 0) % pulse hight
                    NWCP(i,pn) = NWCP1(i)+0.1*MR*(rand()-0.5);
                    NWCP(i,pn) = max([NWCP(i,pn),1]);
                end
                if (i >= (nwd-1)*Ssize*3+Ssize+1) && (i <= (nwd-1)*Ssize*3+Ssize*2) && (Marker_NWC(i,1) ~= 0) % pulse width
                    NWCP(i,pn) = NWCP1(i)+3*MR*(rand()-0.5);
                    NWCP(i,pn) = max([NWCP(i,pn),0.1]);
                    NWCP(i,pn) = min([NWCP(i,pn),1]);
                end
                if (i >= (nwd-1)*Ssize*3+Ssize*2+1) && (i <= (nwd-1)*Ssize*3+Ssize*3) && (Marker_NWC(i,1) ~= 0) % leading edge time
                    NWCP(i,pn) = NWCP1(i)+3*MR*(rand()-0.5);
                    NWCP(i,pn) = max([NWCP(i,pn),0.16]);
                    NWCP(i,pn) = min([NWCP(i,pn),1]);
                end
            end
            if i <= size(NWCP1,1)
                if (Marker_NWC(i,1) * Marker_NWC(i,2) ~= 0) % pos and neg ions with same m/z should be heated equally
                    NWCP(Marker_NWC(i,2),pn) = NWCP(i,pn);
                end
            end
        end
    end
    % Test with board condition
    for bo = 1:BO
       [A,B] = BSGen();
       boardhis(:,:,bo) = A;
       playerlist(bo) = B;
    end
    smallscore = zeros(playerN,1);
    Movehis = zeros(playerN,BO,2);
    matchnum = 0;
    for pn1 = 1:playerN
        PM = 0;
        for bo = 1:BO
            matchnum = matchnum+1;
            smallscoreold = smallscore(pn1,1);
            [S,OESfull,smallscore(pn1,1),movepos] =...
                MakeTest(NWD,nbulk,Tgini,Teini,dt,Ssize,Rsize,PL,SIM,Reactants,SRN,BOLSIGData,...
                Marker_S,Marker_hv,Marker_hv_eff,Sini,inputindex,...
                NWCP(:,pn1),boardhis(:,:,bo),playerlist(bo),smallscore(pn1,1));   
            if smallscore(pn1,1) - smallscoreold >= 10
                PM = PM + 1;
            end
            Shis(:,:,matchnum) = S;
            OEShisfull(:,:,matchnum) = OESfull;
            Movehis(pn1,bo,:) = movepos;
        end
        PMrate(pn1,1) = PM/BO;
    end
    [winnerscore,winner] = max(smallscore);
    scorehis(itML) = smallscore(winner);
    winnerPMrate = PMrate(winner,1);
    %{
    if itML > 100
        scorem1 = mean(scorehis(itML-100+1:itML-50));
        scorem2 = mean(scorehis(itML-50+1:itML));
    end
    %}
    if (winner == 1) %|| ((itML > 100) && (abs(scorem1-scorem2)/scorem2) < 0.005)
        contwin = contwin+1;
        MR = MR*1.001;
        %MR = MR+MRini*0.1;
        MR = min([MR,1]);
    else
        % learn
        contwin = 0;
        NWCP1 = NWCP(:,winner);
        gen = gen+1;
        MR = MRini;
    end

    plot(scorehis(1:itML,1),'color','k','linewidth',1);
    if itML > 10
        hold on;
        plot(movmean(scorehis(1:itML,1),10),'color','g','linewidth',1);
        hold off;
    end
    if itML > 50
        hold on;
        plot(movmean(scorehis(1:itML,1),50),'color','b','linewidth',1);
        hold off;
    end
    if itML > 150
        hold on;
        plot(movmean(scorehis(1:itML,1),150),'color','r','linewidth',1);
        hold off;
    end
    if itML > 500
        hold on;
        plot(movmean(scorehis(1:itML,1),500),'color','y','linewidth',1);
        hold off;
    end
    grid minor;
    title('Top Player Score');
    str = sprintf(('it = %d, gen = %d, score = %d, MR = %d, Best Player = %d, PM Rate = %f'),itML,gen,contwin,MR,winnerscore,winnerPMrate);
    disp(str); %#ok<*DSPS>
    pause(0.001);
    save(strcat(SaveAddress,'PBTicTacToe Results','---',...
        'it',num2str(itML),'.mat'),...
    'Shis','OEShisfull','boardhis','contwin','gen','MR',...
    'NWCP','smallscore','scorehis','PMrate','Movehis');
    
    if contwin > 10^4
        disp('Training Finished');
        break;
    end
end    

disp('All Finished.');
end

function [board,player] = BSGen()
% Board situation generator
deciB = round(19682*rand());
triB = str2double(dec2base(deciB,3));
board = zeros(3,3);
k = 0;
stonenum = [0,0];
for x = 1:3
    for y = 1:3
        k = k+1;
        temp = floor(triB/(10^(9-k)));
        sv = temp-floor(temp/10)*10;
        if sv ~= 0
            stonenum(sv) = stonenum(sv)+1;
            if sum(stonenum) >= 7
                break;
            end
            if stonenum(2)-stonenum(1) == 2
                sv = 1;
                stonenum(2) = stonenum(2)-1;
                stonenum(sv) = stonenum(sv)+1;
            end
            if stonenum(1)-stonenum(2) == 2
                sv = 2;
                stonenum(1) = stonenum(1)-1;
                stonenum(sv) = stonenum(sv)+1;
            end
        end
        board(x,y) = sv;
        victory = findvictory(board);
        if victory ~= 0
            board(x,y) = 0;
        end
    end
    if sum(stonenum) >= 7
        break;
    end
end
[~,player] = min(stonenum);
if rand() > 0.5
    board = flipud(board);
end
if rand() > 0.5
    board = fliplr(board);
end
end

function LMV = TicTacToeAI_Eva(board)

% find legal move
LMV = zeros(9,3);
i = 0;
for x = 1:3
    for y = 1:3
        if board(x,y) == 0
            i = i+1;
            LMV(i,1:2) = [x,y];
        end
    end
end
LMN = i; % the legal move number
LMV = LMV(1:LMN,:);

% Start the evaluation
value = 10;
valuemark = 0;
% 9. Win: If the player has two in a row, they can place a third to get three in a row.
for i = 1:LMN 
    boardtemp = board;
    boardtemp(LMV(i,1),LMV(i,2)) = 1;
    if findvictory(boardtemp) == 1
        if LMV(i,3) == 0
            LMV(i,3) = value;
            valuemark = 1;
        end
    end
end
if valuemark == 1
    valuemark = 0;
    value = value/4;
end

% 8. Block: If the opponent has two in a row, the player must play the third themselves to block the opponent.
for i = 1:LMN
    boardtemp = board;
    boardtemp(LMV(i,1),LMV(i,2)) = 2;
    if findvictory(boardtemp) == 2
        if LMV(i,3) == 0
            LMV(i,3) = value;
            valuemark = 1;
        end
    end
end
if valuemark == 1
    valuemark = 0;
    value = value/4;
end

% 7. Fork: Cause a scenario where the player has two ways to win (two non-blocked lines of 2).
for i = 1:LMN
    status = checkF(board,[LMV(i,1),LMV(i,2)],1);
    if status == 1
        if LMV(i,3) == 0
            LMV(i,3) = value;
            valuemark = 1;
        end
    end
end
if valuemark == 1
    valuemark = 0;
    value = value/4;
end

% 6. Blocking an opponent's fork
OFC = zeros(LMN,1); % opponent's fork choices
for i = 1:LMN
    OFC(i,1) = checkF(board,[LMV(i,1),LMV(i,2)],2);
end
if sum(OFC) == 1
    [~,goodi] = max(OFC);
    if LMV(goodi,3) == 0
        LMV(goodi,3) = value;
        valuemark = 1;
    end
end
if sum(OFC) > 1 % 5. there are multiple possible opponent's fork choices
    % Special Strategy:
    % Instead of simply blocking one of the opponent's fork choices, 
    % make two in a row to compel the opponent to defend,
    % and the opponent's defence will not make his fork.
    % If there is no such a possibility, the game is surely lose.
    for i = 1:LMN
        boardtemp = board;
        boardtemp(LMV(i,1),LMV(i,2)) = 1;
        LM2 = zeros(9,2);
        i2 = 0;
        for x = 1:3
            for y = 1:3
                if boardtemp(x,y) == 0
                    i2 = i2+1;
                    LM2(i2,:) = [x,y];
                end
            end
        end
        LMN2 = i2; % the legal move number remain to check if there is a two-in-row
        score = 0;
        for i2 = 1:LMN2 
            boardtemp2 = boardtemp;
            boardtemp2(LM2(i2,1),LM2(i2,2)) = 1;
            if findvictory(boardtemp2) == 1 
                % This means the move "LM2(i2,1),LM2(i2,2)" can win the game, which means that 
                % "LM(i,1),LM(i,2)" can make a two-in-row.
                % The two-in-row here should force the opponent to
                % defend but such a defend should not cause any
                % other opponent's fork when the opponent blocks it
                if checkF(boardtemp,[LM2(i2,1),LM2(i2,2)],2)  == 0 % The opponent now block the two-in-rwo
                    score = score+1; % The opponent's block will not cause his fork, best move found
                end
                break;
            end
        end
        if score == 1
            if LMV(i,3) == 0
                LMV(i,3) = value;
                valuemark = 1;
            end
        end
    end % The special strategy loop end
end % Multiple OFC "if" end
if valuemark == 1
    valuemark = 0;
    value = value/4;
end

% 4. Center: A player marks the center.
if board(2,2) == 0
    for i = 1:LMN
       if sum(LMV(i,1:2) == [2,2]) == 2
           if LMV(i,3) == 0
                LMV(i,3) = value;
                valuemark = 1;
           end
           break;
       end
    end
end
if valuemark == 1
    valuemark = 0;
    value = value/4;
end
    
% 3. Opposite corner: If the opponent is in the corner, the player plays the opposite corner.
if (board(1,1) == 2) && (board(3,3) == 0)
    for i = 1:LMN
       if sum(LMV(i,1:2) == [3,3]) == 2
           if LMV(i,3) == 0
                LMV(i,3) = value;
                valuemark = 1;
           end
           break;
       end
    end
end
if (board(1,3) == 2) && (board(3,1) == 0)
    for i = 1:LMN
       if sum(LMV(i,1:2) == [3,1]) == 2
           if LMV(i,3) == 0
                LMV(i,3) = value;
                valuemark = 1;
           end
           break;
       end
    end
end
if (board(3,1) == 2) && (board(1,3) == 0)
    for i = 1:LMN
       if sum(LMV(i,1:2) == [1,3]) == 2
           if LMV(i,3) == 0
                LMV(i,3) = value;
                valuemark = 1;
           end
           break;
       end
    end
end
if (board(3,3) == 2) && (board(1,1) == 0)
    for i = 1:LMN
       if sum(LMV(i,1:2) == [1,1]) == 2
           if LMV(i,3) == 0
                LMV(i,3) = value;
                valuemark = 1;
           end
           break;
       end
    end
end
if valuemark == 1
    valuemark = 0;
    value = value/4;
end

% 2. Empty corner: The player plays in a corner square.
for i = 1:LMN
    if (LMV(i,3) == 0) &&...
            ((sum(LMV(i,1:2) == [3,3]) == 2) ||...
            (sum(LMV(i,1:2) == [3,1]) == 2) ||...
            (sum(LMV(i,1:2) == [1,3]) == 2)||...
            (sum(LMV(i,1:2) == [1,1]) == 2))
        LMV(i,3) = value;
        valuemark = 1;
    end
end
if valuemark == 1
    %valuemark = 0;
    value = value/4;
end

% 1. Empty side: The player plays in a middle square on any of the four sides.
for i = 1:LMN
    if (LMV(i,3) == 0) &&...
            ((sum(LMV(i,1:2) == [1,2]) == 2) ||...
            (sum(LMV(i,1:2) == [2,1]) == 2) ||...
            (sum(LMV(i,1:2) == [2,3]) == 2) ||...
            (sum(LMV(i,1:2) == [3,2]) == 2))
        LMV(i,3) = value;
        %valuemark = 1;
    end
end

end

%{
function movepos = TicTacToeAI(board)
RUS = 1; % rate of using the strategy
% find legal move
LM = zeros(9,2);
i = 0;
for x = 1:3
    for y = 1:3
        if board(x,y) == 0
            i = i+1;
            LM(i,:) = [x,y];
        end
    end
end
LMN = i; % the legal move number
if rand() > RUS
    moveindex = ceil(rand()*LMN);
    movepos = LM(moveindex,:);
else
    % Start the strategy
    movepos = 0;
    for i = 1:LMN % Win: If the player has two in a row, they can place a third to get three in a row.
        boardtemp = board;
        boardtemp(LM(i,1),LM(i,2)) = 1;
        if findvictory(boardtemp) == 1
            movepos = LM(i,:);
            break;
        end
    end
    if movepos == 0 % Block: If the opponent has two in a row, the player must play the third themselves to block the opponent.
        for i = 1:LMN
            boardtemp = board;
            boardtemp(LM(i,1),LM(i,2)) = 2;
            if findvictory(boardtemp) == 2
                movepos = LM(i,:);
                break;
            end
        end
    end
    if movepos == 0 % Fork: Cause a scenario where the player has two ways to win (two non-blocked lines of 2).
        for i = 1:LMN
            status = checkF(board,[LM(i,1),LM(i,2)],1);
            if status == 1
                movepos = LM(i,:);
                break;
            end
        end
    end
    if movepos == 0 % Blocking an opponent's fork
        OFC = zeros(LMN,1); % opponent's fork choices
        for i = 1:LMN
            OFC(i,1) = checkF(board,[LM(i,1),LM(i,2)],2);
        end
        if sum(OFC) == 1
            [~,goodi] = max(OFC);
            movepos =LM(goodi,:);
        end
        if sum(OFC) > 1 % there are multiple possible opponent's fork choices
            % Special Strategy:
            % Instead of simply blocking one of the opponent's fork choices, 
            % make two in a row to compel the opponent to defend,
            % and the opponent's defence will not make his fork.
            % If there is no such a possibility, the game is surely lose.
            for i = 1:LMN
                boardtemp = board;
                boardtemp(LM(i,1),LM(i,2)) = 1;
                LM2 = zeros(9,2);
                i2 = 0;
                for x = 1:3
                    for y = 1:3
                        if boardtemp(x,y) == 0
                            i2 = i2+1;
                            LM2(i2,:) = [x,y];
                        end
                    end
                end
                LMN2 = i2; % the legal move number remain to check if there is a two-in-row
                score = 0;
                for i2 = 1:LMN2 
                    boardtemp2 = boardtemp;
                    boardtemp2(LM2(i2,1),LM2(i2,2)) = 1;
                    if findvictory(boardtemp2) == 1 
                        % This means the move "LM2(i2,1),LM2(i2,2)" can win the game, which means that 
                        % "LM(i,1),LM(i,2)" can make a two-in-row.
                        % The two-in-row here should force the opponent to
                        % defend but such a defend should not cause any
                        % other opponent's fork when the opponent blocks it
                        if checkF(boardtemp,[LM2(i2,1),LM2(i2,2)],2)  == 0 % The opponent now block the two-in-rwo
                            score = score+1; % The opponent's block will not cause his fork, best move found
                        end
                        break;
                    end
                end
                if score == 1
                    movepos = LM(i,:);
                    break;
                end
            end % The special strategy loop end
        end % Multiple OFC "if" end
    end % block the opponent's fork "if" end
    if movepos == 0 % Center: A player marks the center.
        if board(2,2) == 0
            movepos = [2,2];
        end
    end
    if movepos == 0 % Opposite corner: If the opponent is in the corner, the player plays the opposite corner.
        if (board(1,1) == 2) && (board(3,3) == 0)
            movepos = [3,3];
        end
        if (board(1,3) == 2) && (board(3,1) == 0)
            movepos = [3,1];
        end
        if (board(3,1) == 2) && (board(1,3) == 0)
            movepos = [1,3];
        end
        if (board(3,3) == 2) && (board(1,1) == 0)
            movepos = [1,1];
        end
    end
    if movepos == 0 % Empty corner: The player plays in a corner square.
        if (board(3,3) == 0)
            movepos = [3,3];
        end
        if (board(3,1) == 0)
            movepos = [3,1];
        end
        if (board(1,3) == 0)
            movepos = [1,3];
        end
        if (board(1,1) == 0)
            movepos = [1,1];
        end
    end
    if movepos == 0 % Empty side: The player plays in a middle square on any of the four sides.
        if (board(1,2) == 0)
            movepos = [1,2];
        end
        if (board(2,1) == 0)
            movepos = [2,1];
        end
        if (board(2,3) == 0)
            movepos = [2,3];
        end
        if (board(3,2) == 0)
            movepos = [3,2];
        end
    end
end
end
%}

function status = checkF(board,pos2mov,player)
% This is to check if the move "pos2mov" will cause a fork
% status = 1 for causing a fork
status = 0;
status1 = 0;
status2 = 0;
boardtemp = board;
playern = 0;
for i = 1:3
    for j = 1:3
        if board(i,j) == player
            playern = playern+1;
        end
    end
end
boardtemp(pos2mov(1),pos2mov(2)) = player;
if (findvictory(boardtemp) ~= player) && (playern >= 2)
    % check if boardtemp has 2 two-in-row:
    % find remaining legal move
    LM2 = zeros(9,2);
    i2 = 0;
    for x = 1:3
        for y = 1:3
            if boardtemp(x,y) == 0
                i2 = i2+1;
                LM2(i2,:) = [x,y];
            end
        end
    end
    LMN2 = i2; % the legal move number
    for i2 = 1:LMN2 % Win
        boardtemp2 = boardtemp;
        boardtemp2(LM2(i2,1),LM2(i2,2)) = player;
        if (findvictory(boardtemp2) == player) 
            status1 = status1+1;
        end
    end
    if status1 >= 2
        % check if board has 2 two-in-row:
        % find remaining legal move
        LM2 = zeros(9,2);
        i2 = 0;
        for x = 1:3
            for y = 1:3
                if board(x,y) == 0
                    i2 = i2+1;
                    LM2(i2,:) = [x,y];
                end
            end
        end
        LMN2 = i2; % the legal move number
        for i2 = 1:LMN2 % Win
            boardtemp3 = board;
            boardtemp3(LM2(i2,1),LM2(i2,2)) = player;
            if (findvictory(boardtemp3) == player) 
                status2 = status2+1;
            end
        end
        if status1-status2 >= 2
            status = 1;
        end
    end
end
end

function [Shis,OESfull,score,movepos] = ...
    MakeTest(NWD,nbulkini,Tgini,Teini,dt,Ssize,Rsize,PL,SIM,Reactants,SRN,BOLSIGData,...
    Marker_S,Marker_hv,Marker_hv_eff,Sini,inputindex,NWCPini,board,player,score)
% Set the board for player
if player ~= 1
    boardtemp = board;
    for x = 1:3
        for y = 1:3
            if boardtemp(x,y) == 2
                boardtemp(x,y) = -1;
            end
        end
    end
    boardtemp = boardtemp*(-1);
    for x = 1:3
        for y = 1:3
            if boardtemp(x,y) == -1
                boardtemp(x,y) = 2;
            end
        end
    end 
    board = boardtemp;
end
S = move2S(board,1,Sini,inputindex,Marker_S);

% Preparing Plasma
NWD1 = NWD/9;
NWCP1 = zeros(size(NWCPini,1)/9,9);
PP = zeros(Ssize,NWD1,9); % species temperature alteration pulse info 
TeList = ones(PL,9)*Teini;
NWCCList = ones(Ssize,PL,9);
for pn = 1:9
    NWCP1(:,pn) = NWCPini((pn-1)*Ssize*3+1:(pn-1)*Ssize*3+Ssize*3*NWD1,1);
    for sn = 1:Ssize
        for nwd = 1:NWD1
            PP(sn,(nwd-1)*3+1,pn) = NWCP1(sn+Ssize+(nwd-1)*3*Ssize,pn)*PL/NWD1+(nwd-1)*PL/NWD1; % 1st col: leading edge time [in a real time unit, such as us]
            PP(sn,(nwd-1)*3+2,pn) = NWCP1(sn+Ssize+(nwd-1)*3*Ssize,pn)*PL/NWD1+(nwd-1)*PL/NWD1+...
                NWCP1(sn+Ssize*2+(nwd-1)*3*Ssize,pn)*PL/NWD1; % 2nd col: falling edge time [in a real time unit, such as us]
            if PP(sn,(nwd-1)*3+2,pn) > PL
                PP(sn,(nwd-1)*3+2,pn) = PL;
            end
            PP(sn,(nwd-1)*3+3,pn) = NWCP1(sn+(nwd-1)*3*Ssize,pn); % 3rd col: peak height [in pultiplier (no unit)]
        end
    end
end
for pn = 1:9
    for tn = 1:PL
        for nwd = 1:NWD1
            if (tn >= PP(Marker_S.e,(nwd-1)*3+1,pn)) && (tn <= PP(Marker_S.e,(nwd-1)*3+2,pn))
                TeList(tn,pn) = Teini* PP(Marker_S.e,(nwd-1)*3+3,pn);
            end
            for sn = 1:Ssize
                if (tn >= PP(sn,(nwd-1)*3+1,pn)) && (tn <= PP(sn,(nwd-1)*3+2,pn))
                    NWCCList(sn,tn,pn) = PP(sn,(nwd-1)*3+3,pn);
                end
            end
        end
    end
end
OES = zeros(9,1);
Shis = zeros(Ssize,pn);
OESfull = zeros(size(Marker_hv,1),pn);
for pn = 1:9
    Tg = Tgini*mean(NWCCList(:,:,pn),2)'*(S/sum(S));
    [~,Shist,~,hvfullhist,~,~] = ...
        ZDCS(TeList(:,pn),Tg,nbulkini,dt,S,Marker_S.e,Marker_S.He,Marker_S.M,Marker_hv,Marker_hv_eff,...
        Ssize,Rsize,PL,SIM,Reactants,SRN,BOLSIGData,NWCCList(:,:,pn));
    Shis(:,pn) = mean(Shist,2);
    OESfull(:,pn) = mean(hvfullhist(:,end-1:end),2);
    OES(pn) = OESfull(10,pn);
end
movepos = OES2move(board,OES); % plasma's move
LMV = TicTacToeAI_Eva(board); % evaluate the board
if movepos(1) ~= -1
    for i = 1:size(LMV,1)
        if movepos == LMV(i,1:2)
            score = score+LMV(i,3);
        end
    end
end
score = score+0.1*size(LMV,1);
end

%{
function [Shishis,OEShisfullhis,boardhishis,scoresmall] = ...
    MatchBegin(nbulkini,Tgini,Teini,dt,BO,Ssize,Rsize,PL,SIM,Reactants,SRN,BOLSIGData,...
    OESRef,Marker_S,Marker_hv,Marker_hv_eff,Sini,inputindex,NWCP1)
scoresmall = zeros(2,1);
boardhishis = zeros(3,3,9,BO);
Shishis = zeros(Ssize,9,BO);
OEShisfullhis = zeros(size(Marker_hv,1),9,BO);
for roundN = 1:BO 
    board = zeros(3,3);
    boardhis = zeros(3,3,9);
    Shis = zeros(Ssize,9);
    OEShis = zeros(size(Marker_hv_eff,1),9);
    OEShisfull = zeros(size(Marker_hv,1),9);
    S = Sini;
    %extraS = zeros(2,1);
    for nmove = 1:9
        nbulk = nbulkini;
        if mod(nmove,2) == mod(roundN,2)
            % Preparing Plasma 1
            %titlelist = {'T_NO','T_N2O','T_H2O','T_N2','T_O3','T_He','T_O2','T_H2','T_NO2','Te'};
            Te = Teini*NWCP1(Marker_S.e,1);
            TeList = Te*ones(PL,1);
            Tg = Tgini*NWCP1'*(S/sum(S));
            [~,~,hvhist,hvfullhist,~,~] = ...
                ZDCS(TeList,Tg,nbulk,dt,S,Marker_S.e,Marker_S.He,Marker_S.M,Marker_hv,Marker_hv_eff,...
                Ssize,Rsize,PL,SIM,Reactants,SRN,BOLSIGData,NWCP1);
            OES = mean(hvhist,2);
            OESfull = mean(hvfullhist,2);
            OES = abs(OES-OESRef)./OESRef;
            movepos = OES2move(board,OES);
            board = makemove(movepos,board,1);
        else
            % Preparing Plasma 2
            boardtemp = board;
            for x = 1:3
                for y = 1:3
                    if boardtemp(x,y) == 2
                        boardtemp(x,y) = -1;
                    end
                end
            end
            boardtemp = boardtemp*(-1);
            for x = 1:3
                for y = 1:3
                    if boardtemp(x,y) == -1
                        boardtemp(x,y) = 2;
                    end
                end
            end
            movepos = TicTacToeAI(boardtemp);
            board = makemove(movepos,board,2);
            S = move2S(board,1,Sini,inputindex,Marker_S); % the next input
        end
        Shis(:,nmove) = S;
        OEShis(:,nmove) = OES;
        OEShisfull(:,nmove) = OESfull; %#ok<*AGROW>
        boardhis(:,:,nmove) = board;
        victory = findvictory(board);
        if victory ~= 0
            scoresmall(victory) = scoresmall(victory)+10;
            break;
        end
    end % nmove loop end
    scoresmall(mod(roundN,2)+1) = scoresmall(mod(roundN,2)+1)+nmove; % The 2nd move player gain more score if survive longer
    if victory ~= 0 % if someone win
        scoresmall(victory) = scoresmall(victory)+(9-nmove); % The winner gain more score for a quicker win
    end
    %scoresmall = scoresmall+extraS;
    Shishis(:,:,roundN) = Shis;
    OEShisfullhis(:,:,roundN) = OEShisfull;
    boardhishis(:,:,1:size(boardhis,3),roundN) = boardhis;
end % round loop end
end
%}

function victory = findvictory(board)
victory = 0;
for x = 1:3
    for y = 1:3
        if x == 2
            if y == 2
                if (board(x-1,y-1) == board(x,y)) && (board(x+1,y+1) == board(x,y))
                    victory = board(x,y);
                end
                if (board(x+1,y-1) == board(x,y)) && (board(x-1,y+1) == board(x,y))
                    victory = board(x,y);
                end
                if (board(x+1,y) == board(x,y)) && (board(x-1,y) == board(x,y))
                    victory = board(x,y);
                end
                if (board(x,y-1) == board(x,y)) && (board(x,y+1) == board(x,y))
                    victory = board(x,y);
                end
            else
                if (board(x+1,y) == board(x,y)) && (board(x-1,y) == board(x,y))
                    victory = board(x,y);
                end
            end
        end
        if y == 2
            if (board(x,y-1) == board(x,y)) && (board(x,y+1) == board(x,y))
                victory = board(x,y);
            end
        end
        if victory ~= 0
            break;
        end
    end
    if victory ~= 0
        break;
    end
end
end

function S = move2S(board,player,Sini,inputindex,Marker_S)
weight = 100;
Smod = ones(size(Sini,1),1);
for x = 1:3
    for y = 1:3
        if board(x,y) == 0
            Smod(inputindex((x-1)*3+y)) = 1;
        else
            if board(x,y) == player
                Smod(inputindex((x-1)*3+y)) = 1/weight;
            else
                Smod(inputindex((x-1)*3+y))  = 1*weight;
            end
        end
    end
end
S = Sini.*Smod;
S(Marker_S.M) = sum(S);
end

%{
function board = makemove(movepos,board,player)
board(movepos(1),movepos(2)) = player;
end
%}

function [movepos] = OES2move(board,OES)
OES = OES*10^(-12);
rs = zeros(9,2);
% rs(:,1) = 1:1:size(rs,1); % this line was used before v10f
rs(:,1) = [2,9,4,7,5,3,6,8,1];
rs(:,2) = OES;
rssort = sortrows(rs,2,'descend');
for i = 1:size(rssort,1)
    y = mod(rssort(i,1)-1,3)+1;
    x = (rssort(i,1)-y)/3+1;
    if board(x,y) == 0
        movepos = [x,y];
        break;
    end
end
end

function [SS,Ssize,Rsize,SIM,Reactants,SRN,...
    Marker_S,Marker_hv,Marker_hv_eff,Marker_es,Marker_eval,Marker_ion] =...
    LoadReactions(ReactionFile)
RList = importdata(ReactionFile);
Rsize = size(RList,1);
% Find Species
SS = cell(1000,1);
Ssize = 1;
for i = 1:Rsize
    temp = RList{i};
    temp1 = split(temp,' ');
    for j = 1:size(temp1,1)
        if (strcmp(temp1{j},'+') == 0) && (strcmp(temp1{j},'=>') == 0) 
            temp2 = split(temp1{j},'');
            temp3 = temp2{2};
            if isnan(str2double(temp3)) == true
                flag = 0;
                for k = 1:Ssize
                    if isempty(SS{k}) == 0
                        if strcmp(SS{k},temp1{j}) == 1
                            flag = 1;
                        end
                    end
                end
                if flag == 0
                    SS{Ssize} = temp1{j};
                    Ssize = Ssize+1;
                end
            end
        end
    end
end
Ssize = Ssize-1;
SS = SS(1:Ssize,1);

% Find SIM
SIM = zeros(Rsize,Ssize);
for i = 1:Rsize
    temp = RList{i};
    temp1 = split(temp,' => ');
    tempR = split(temp1{1},' + ');
    tempP = split(temp1{2},' + ');
    % Reactants
    for j = 1:size(tempR,1)
        tempRS = tempR{j};
        tempCO0 = split(tempRS,'');
        tempCO = tempCO0{2};
        if isnan(str2double(tempCO)) == true
            CO = 1;
            tempRS2 = tempRS;
        else
            CO = str2double(tempCO);
            tempRS2 = strcat(tempCO0{3:size(tempCO0,1)-1});
        end
        for sn = 1:Ssize
            if strcmp(SS{sn},tempRS2)
                SIM(i,sn) = SIM(i,sn)-CO;
            end
        end
    end
    % Products
    for j = 1:size(tempP,1)
        tempPS = tempP{j};
        tempCO0 = split(tempPS,'');
        tempCO = tempCO0{2};
        if isnan(str2double(tempCO)) == true
            CO = 1;
            tempPS2 = tempPS;
        else
            CO = str2double(tempCO);
            tempPS2 = strcat(tempCO0{3:size(tempCO0,1)-1});
        end
        for sn = 1:Ssize
            if strcmp(SS{sn},tempPS2)
                SIM(i,sn) = SIM(i,sn)+CO;
            end
        end
    end
end
[Marker_S,Marker_hv,Marker_hv_eff,Marker_es,Marker_eval,Marker_ion] = FindMarker(SS,Ssize);

% Find Reactants
RIM = zeros(Rsize,Ssize); % Find the reactants power
for i = 1:Rsize
    temp = RList{i};
    temp1 = split(temp,' => ');
    tempR = split(temp1{1},' + ');
    for j = 1:size(tempR,1)
        tempRS = tempR{j};
        tempCO0 = split(tempRS,'');
        tempCO = tempCO0{2};
        if isnan(str2double(tempCO)) == true
            c = tempR{j};
        else
            c = tempCO0{3:size(tempCO0,1)-1};
        end
        for k = 1:Ssize
            if strcmp(SS{k},c)
                if isnan(str2double(tempCO)) == true
                    CO = 1;
                else
                    CO = str2double(tempCO);
                end
                RIM(i,k) = CO;
                break;
            end
        end
    end
end
[SIM,RIM,Rsize] = AdditionalReaction(SS,SIM,RIM,Marker_hv);
Reactants = zeros(Rsize,3);
for rn = 1:Rsize
    k = 0;
    for sn = 1:Ssize
        switch RIM(rn,sn)
            case 1
                k = k+1;
                Reactants(rn,k) = sn;
            case 2
                k = k+1;
                Reactants(rn,k) = sn;
                k = k+1;
                Reactants(rn,k) = sn;
        end
    end
end
SRN = zeros(Ssize,1); % SpeciesReactantsNum
for sn = 1:Ssize
    for eqn = 1:Rsize
        for k = 1:3
            if Reactants(eqn,k) == sn
                SRN(sn) = SRN(sn) + 1;
            end
        end
    end
end
end

function [SIM,RIM,Rsize] = AdditionalReaction(SS,SIM,RIM,Marker_hv)
Ssize = size(SIM,2);
Rsize = size(SIM,1);
List = {'H-','H','e';
    'O-','O','e';
    'O2-','O2','e';
    'O3-','O3','e';
    'OH-','OH','e';
    'H','H+','e';
    'H2','H2+','e';
    'N','N+','e';
    'NO','NO+','e';
    'O','O+','e';
    'O2','O2+','e';
    'N2','N2+','e';
    'HNO2','HNO2','';
    'HNO3','HNO2','';
    'N2O5','N2O5','';
    'NO2','NO2','';
    'NO3','NO3','';
    'O3','O3',''};
SIMtemp = zeros(Rsize+size(List,1)*size(Marker_hv,1),Ssize);
SIMtemp(1:Rsize,:) = SIM;
RIMtemp = zeros(Rsize+size(List,1)*size(Marker_hv,1),Ssize);
RIMtemp(1:Rsize,:) = RIM;
r = Rsize+1;
for hv = 1:size(Marker_hv,1)
   for i = 1:size(List,1)
       for s = 1:Ssize
            if strcmp(List{i,1},SS{s})
                SIMtemp(r,s) = SIMtemp(r,s)-1;
                RIMtemp(r,s) = RIMtemp(r,s)+1;
            end
            if strcmp(List{i,2},SS{s})
                SIMtemp(r,s) = SIMtemp(r,s)+1;
            end
            if strcmp(List{i,3},SS{s})
                SIMtemp(r,s) = SIMtemp(r,s)+1;
            end
        end
        r = r+1;
    end
end
SIM = SIMtemp;
RIM = RIMtemp;
Rsize = size(SIM,1);
end

function [Marker_S,Marker_hv,Marker_hv_eff,Marker_es,Marker_eval,Marker_ion] = FindMarker(SS,Ssize)
Marker_hv = zeros(100,2);
Marker_es = zeros(100,1);
Marker_eval = zeros(100,1);
Marker_ion = zeros(100,2);
hvi = 0;
esi = 0;
evali = 0;
ionk = 0;
for sn = 1:Ssize
    temp = SS{sn};
    if strcmp(temp,'e') || strcmp(temp,'He+') || strcmp(temp,'He2+') ||...
            strcmp(temp,'H+') || strcmp(temp,'H') || strcmp(temp,'H2+') ||...
            strcmp(temp,'H2') || strcmp(temp,'N+') || strcmp(temp,'N') ||...
            strcmp(temp,'N2+') || strcmp(temp,'O+') || strcmp(temp,'O') ||...
            strcmp(temp,'O-') || strcmp(temp,'O2+') || strcmp(temp,'O3') ||...
            strcmp(temp,'O3-') || strcmp(temp,'H2O+') || strcmp(temp,'OH') ||...
            strcmp(temp,'H-') || strcmp(temp,'OH+') || strcmp(temp,'OH-') ||...
            strcmp(temp,'NO+') || strcmp(temp,'NO') || strcmp(temp,'NO-') ||...
            strcmp(temp,'NO2+') || strcmp(temp,'NO2') || strcmp(temp,'NO2-') ||...
            strcmp(temp,'N2O+') || strcmp(temp,'N2O') || strcmp(temp,'HNO3') ||...
            strcmp(temp,'N2O-') || strcmp(temp,'NO3-') || strcmp(temp,'NO3') ||...
            strcmp(temp,'HNO') || strcmp(temp,'HO2') || strcmp(temp,'H2O2') ||...
            strcmp(temp,'HNO2') || strcmp(temp,'N2O5')
        evali = evali+1;
        Marker_eval(evali) = sn;
    end
    % Find ions
    temp2p = strsplit(temp,'+');
    temp2n = strsplit(temp,'-');
    if (strcmp(temp2p{1},temp) == 0) || (strcmp(temp2n{1},temp) == 0)
        ionk = ionk+1;
        if (strcmp(temp2p{1},temp) == 0)
            Marker_ion(ionk,1) = sn;
        else
            Marker_ion(ionk,2) = sn;
        end
    end
    if strcmp(temp,'e')
       Marker_S.e = sn;
    end
    if strcmp(temp,'He')
       Marker_S.He = sn;
    end
    if strcmp(temp,'N2')
       Marker_S.N2 = sn;
    end
    if strcmp(temp,'O2')
       Marker_S.O2 = sn;
    end
    if strcmp(temp,'H2O')
       Marker_S.H2O = sn;
    end
    if strcmp(temp,'M')
       Marker_S.M = sn;
    end
    temp_ini = split(temp,'(');
    if strcmp(temp_ini{1},'hv')
       hvi = hvi+1;
       Marker_hv(hvi,1) = sn;
       temp2 = split(temp_ini{2},'n');
       WL = str2double(temp2{1}); % Wavelength [nm]
       Marker_hv(hvi,2) = WL;
    end
    if (strcmp(temp_ini{1},'He') || strcmp(temp_ini{1},'N2')) && (size(temp_ini,1) == 2)
        esi = esi+1;
        Marker_es(esi,1) = sn;
    end
end
Marker_hv = Marker_hv(1:hvi,:);
Marker_hv = sortrows(Marker_hv,2);
Marker_ion = Marker_ion(1:ionk,:);

% efflist suggestions:
% N2(C): 
%   315.93nm N2(C3?u => B3?g, v' = 1, v" = 0)
%   337.13nm N2(C3?u => B3?g, v' = 0, v" = 0)
% N2+(B): 
%   391.44nm N2+(B2?u+ => X2?g+, v' = 0, v" = 0)
%   427.81nm N2+(B2?u+ => X2?g+, v' = 0, v" = 1)
% He:
%   706.52nm He(33S => 23P)
%   667.82nm He(31D => 21P)
% O:
%   777.4nm O(35P => 35S)
% OH:
%   308.9nm OH(A2?+ => X2?, v' = 0, v" = 0)
%efflist = [308.9;315.93;337.13;391.44;427.81;667.82;706.52;777.4];
%efflist = [308.9;337.13;391.44;706.52;777.4];
%efflist = [308.9;777.4];
%efflist = [728.1400;315.9300];
efflist = [728.14;706.52;58.43;1083;2058.7;391.44;427.81;337.13;315.93];
Marker_hv_eff = zeros(size(efflist,1),2);
j = 1;
for i = 1:size(Marker_hv,1)
    for k = 1:size(efflist,1)
        if Marker_hv(i,2) == efflist(k)
            Marker_hv_eff(j,:) = Marker_hv(i,:);
            j = j+1;
        end
    end
end
Marker_es = Marker_es(1:esi);
Marker_eval = Marker_eval(1:evali);
end

function [S,Shist,hvhist,hvfullhist,RRMhist,RChist] = ...
    ZDCS(TeList,Tg,nbulk,dt,S,emark,Hemark,MMark,Marker_hv,Marker_hv_eff,...
    Ssize,Rsize,tsize,SIM,Reactants,SRN,BOLSIGData,NWCCList)
% Te in [eV]
% Tg in [K]
% Scientific Constants
kB = 1.38*10^(-23); % Boltzmann constant [J/K] 
e = 1.6*10^(-19); % Unit charge [C]
%P = 101325; % Gas bulk pressure [Pa]
%nbulk = P/Tg/kB/1000000; % Gas bulk density [cm-3]

% Initialization
Shist = zeros(Ssize,tsize);
hvhist = zeros(size(Marker_hv_eff,1),tsize);
hvfullhist = zeros(size(Marker_hv,1),tsize);
RRMhist = zeros(Rsize,tsize);
RChist = zeros(Rsize,tsize);
hvold = zeros(size(Marker_hv_eff,1),1);
%estemp = zeros(size(Marker_es,1),1);

for tn = 1:tsize
    Te = TeList(tn);
    ne = S(emark);
    nHe = S(Hemark);
    NWCCvalues = NWCCList(:,tn);
    NWCCapp = ones(Rsize,3);
    for ri = 1:Rsize
       for si = 1:3
           if Reactants(ri,si) ~= 0
                NWCCapp(ri,si) = NWCCvalues(Reactants(ri,si));
           end
       end
    end
    [RRM,RC] = ...
        RRMProcessor(Reactants,SRN,Rsize,S,Te*e/kB,nHe,ne,...
        BOLSIGData,kB,e,Tg,nbulk,dt,NWCCapp);
    S = S+SIM'*squeeze(RRM)*dt; 
    S(MMark) = sum(S);
    
    for sn = 1:size(S,1)
        if S(sn) < 0
            S(sn) = 0;
        end
    end
    % Remove hv
    if tn > 1
        S(Marker_hv(:,1)) = S(Marker_hv(:,1))-hvold;
        for i = 1:size(Marker_hv,1)
            if S(Marker_hv(i,1)) < 0
                S(Marker_hv(i,1)) = 0;
            end
        end
    end
    hvold = S(Marker_hv(:,1));
    %{
    for i = 1:size(Marker_hv_eff,1)
       hvhist(i,tn) = S(Marker_hv_eff(i,1)); 
    end
    %}
    for i = 1:size(Marker_hv,1)
       hvfullhist(i,tn) = S(Marker_hv(i,1)); 
    end
    Shist(:,tn) = S;
    RRMhist(:,tn) = RRM;
    RChist(:,tn) = RC;
end
end

function [RRM,RC] = ...
    RRMProcessor(Reactants,SRN,Rsize,S,Te,nHe,ne,...
    BOLSIGData,kB,e,Tg,nbulk,dt,NWCC)
% All input length unit in [cm] including E and BOLSIGData
% Te and Tg input in [K]
% 2D linear interpolation
rnHe = nHe/nbulk;
rnHe = max([rnHe,0]);
rnHe = min([rnHe,1]);
rne = (ne-10^6)/(10^14-10^6);
rne = max([rne,0]);
rne = min([rne,1]);
T = zeros(size(BOLSIGData,1),size(BOLSIGData,2),2);
for t = 1:2
    T(:,:,t) = (BOLSIGData(:,:,(t-1)*2+2)-BOLSIGData(:,:,(t-1)*2+1))*rne...
        +BOLSIGData(:,:,(t-1)*2+1);
end
U = squeeze((T(:,:,2)-T(:,:,1))*rnHe+T(:,:,1));
U(:,3:size(U,2)) = U(:,3:size(U,2)).*ones(size(U,1),size(U,2)-2);
BOLSIGChart = U; 
BOLSIGChart(:,1:2) = BOLSIGData(:,1:2,1);
RC = RCList(BOLSIGChart,Te,Rsize,kB,e,Tg);
RRMtemp = ones(Rsize,1);
Slimdata = zeros(Rsize,3);
for eq = 1:Rsize
    for k = 1:3
        if Reactants(eq,k) ~= 0
            RRMtemp(eq,1) = RRMtemp(eq,1)*S(Reactants(eq,k)); 
            Slimdata(eq,k) = S(Reactants(eq,k));
        else
            Slimdata(eq,k) = NaN;
        end
    end
end
% NWCC is added here
RRMtemp = RRMtemp.*NWCC(:,1).*NWCC(:,2).*NWCC(:,3);
[Slim,Slimi] = min(Slimdata,[],2);
RRM = RRMtemp.*RC;
for eqn = 1:Rsize
    RRM(eqn) = min([RRM(eqn),...
        Slim(eqn)/dt/SRN(Reactants(eqn,Slimi(eqn)))]);
end
end

function RC = RCList(BOLSIGChart,Te,Rsize,kB,e,Tg)
% All input length unit in [cm] including BOLSIGData
% Te and Tg in [K]
me = 9*10^(-31);
Csize = size(BOLSIGChart,2)-2;
mark = [0,0];
for i = 1:size(BOLSIGChart,1)
    if (Te*kB/e < BOLSIGChart(i,2)) && (mark(1,1) == 0)
        mark = [i,1];
        break;
    end
    if (abs(Te*kB/e-BOLSIGChart(i,2)) < 0.001) && (mark(1,1) == 0)
        mark = [i,0];
        break;
    end
end
if (Te*kB/e) < min(BOLSIGChart(:,2))
    mark = [1,0];
end
if (Te*kB/e) > max(BOLSIGChart(:,2))
    mark = [size(BOLSIGChart,1),0];
end
P = BOLSIGChart(1,3:Csize+2);
if mark(1,2) == 1
    r = (Te*kB/e-BOLSIGChart(mark(1,1)-1,2))/(BOLSIGChart(mark(1,1),2)-BOLSIGChart(mark(1,1)-1,2));
    P = (BOLSIGChart(mark(1,1),3:Csize+2)-BOLSIGChart(mark(1,1)-1,3:Csize+2))*r+...
        BOLSIGChart(mark(1,1)-1,3:Csize+2);
end
if (mark(1,2) == 0) && (mark(1,1) > 0)
    P = BOLSIGChart(mark(1,1),3:Csize+2);
end
% p is the reaction number in BOLSIGData 
% excluding the 1st and 2nd row of E/N and <Te>
% To speed up the code running, using Reduced BOLSIGData: 
% p_reduced(1-18) = p_full(2-19), 
% p_reduced(19-25) = p_full(51,53,66,85,88,99,112)
RC = zeros(Rsize,1);
RC(	1	)=	2.7*10^-13*(Te*kB/e)^-0.75	;	%	e + He+ => He		
RC(	2	)=	1.1*10^-14*Te^-4.5*Tg^2.3	;	%	2e + He+ => He(21P) + e		
RC(	3	)=	1.1*10^-14*Te^-4.5*Tg^2.3	;	%	2e + He+ => He(23P) + e		
RC(	4	)=	1.1*10^-14*Te^-4.5*Tg^2.3	;	%	2e + He+ => He(21S) + e		
RC(	5	)=	1.1*10^-14*Te^-4.5*Tg^2.3	;	%	2e + He+ => He(23S) + e		
RC(	6	)=	P(1)	;	%	e + He => He(23S) + e	*exp(-(Te-229797.1014)^2/(Tg-300)^5)	
RC(	7	)=	P(2)	;	%	e + He => He(21S) + e	*exp(-(Te-239072.4638)^2/(Tg-300)^5)	
RC(	8	)=	P(3)	;	%	e + He => He(23P) + e	*exp(-(Te-243014.4928)^2/(Tg-300)^5)	
RC(	9	)=	P(4)	;	%	e + He => He(21P) + e	*exp(-(Te-246028.9855)^2/(Tg-300)^5)	
RC(	10	)=	P(5)	;	%	e + He => He(33S) + e	*exp(-(Te-263420.2899)^2/(Tg-300)^5)	
RC(	11	)=	P(6)	;	%	e + He => He(31S) + e	*exp(-(Te-265739.1304)^2/(Tg-300)^5)	
RC(	12	)=	P(7)	;	%	e + He => He(33P) + e	*exp(-(Te-266782.6087)^2/(Tg-300)^5)	
RC(	13	)=	P(8)	;	%	e + He => He(33D) + e	*exp(-(Te-267478.2609)^2/(Tg-300)^5)	
RC(	14	)=	P(10)	;	%	e + He => He(31P) + e	*exp(-(Te-267675.3623)^2/(Tg-300)^5)	
RC(	15	)=	P(9)	;	%	e + He => He(31D) + e	*exp(-(Te-267524.6377)^2/(Tg-300)^5)	
RC(	16	)=	P(11)	;	%	e + He => He(43S) + e	*exp(-(Te-273542.029)^2/(Tg-300)^5)	
RC(	17	)=	P(12)	;	%	e + He => He(41S) + e	*exp(-(Te-274481.1594)^2/(Tg-300)^5)	
RC(	18	)=	P(13)	;	%	e + He => He(43P) + e	*exp(-(Te-274875.3623)^2/(Tg-300)^5)	
RC(	19	)=	P(14)	;	%	e + He => He(41D) + e	*exp(-(Te-275200)^2/(Tg-300)^5)	
RC(	20	)=	P(15)	;	%	e + He => He(43D) + e	*exp(-(Te-275200)^2/(Tg-300)^5)	
RC(	21	)=	P(16)	;	%	e + He => He(41F) + e	*exp(-(Te-275211.5942)^2/(Tg-300)^5)	
RC(	22	)=	P(17)	;	%	e + He => He(43F) + e	*exp(-(Te-275211.5942)^2/(Tg-300)^5)	
RC(	23	)=	P(18)	;	%	e + He => He(41P) + e	*exp(-(Te-275269.5652)^2/(Tg-300)^5)	
RC(	24	)=	P(19)	;	%	e + He => He+ + 2e		
RC(	25	)=	1.28*10^-7*(Te*kB/e)^0.6*exp(-4.78/(Te*kB/e))	;	%	e + He(21P) => He+ + 2e		
RC(	26	)=	1.28*10^-7*(Te*kB/e)^0.6*exp(-4.78/(Te*kB/e))	;	%	e + He(23P) => He+ + 2e		
RC(	27	)=	1.28*10^-7*(Te*kB/e)^0.6*exp(-4.78/(Te*kB/e))	;	%	e + He(21S) => He+ + 2e		
RC(	28	)=	1.28*10^-7*(Te*kB/e)^0.6*exp(-4.78/(Te*kB/e))	;	%	e + He(23S) => He+ + 2e		
RC(	29	)=	1.8*10^-7*(1+0.47*(Te*kB/e))^-1	;	%	e + He(21P) => He(23P) + e		
RC(	30	)=	3.7*10^-7*(1+0.37*(Te*kB/e))^-1	;	%	e + He(21P) => He(21S) + e		
RC(	31	)=	3.1*10^-8*(1+0.13*(Te*kB/e)^2)^-1	;	%	e + He(21P) => He(23S) + e		
RC(	32	)=	5*10^-10	;	%	e + He(21P) => He + e		
RC(	33	)=	3.1*10^-8*(1+0.94*(Te*kB/e))^-1	;	%	e + He(23P) => He(21S) + e		
RC(	34	)=	2.2*10^-7*(Te*kB/e)^(1/3)	;	%	e + He(23P) => He(23S) + e		
RC(	35	)=	2.4*10^-10	;	%	e + He(23P) => He + e		
RC(	36	)=	3.8*10^-7*(1+Te*kB/e)^-1	;	%	e + He(21S) => He(23S) + e		
RC(	37	)=	3*10^-9	;	%	e + He(21S) => He + e		
RC(	38	)=	4.1*10^-9*(1+Te*kB/e)^-1	;	%	e + He(23S) => He + e		
RC(	39	)=	8.9*10^-9*(Tg/Te)^1.5	;	%	e + He2+ => He(21P) + He		
RC(	40	)=	8.9*10^-9*(Tg/Te)^1.5	;	%	e + He2+ => He(23P) + He		
RC(	41	)=	8.9*10^-9*(Tg/Te)^1.5	;	%	e + He2+ => He(21S) + He		
RC(	42	)=	8.9*10^-9*(Tg/Te)^1.5	;	%	e + He2+ => He(23S) + He		
RC(	43	)=	10^-26*(Tg/Te)^2	;	%	e + He + He2+ => He(21P) + 2He		
RC(	44	)=	10^-26*(Tg/Te)^2	;	%	e + He + He2+ => He(23P) + 2He		
RC(	45	)=	10^-26*(Tg/Te)^2	;	%	e + He + He2+ => He(23S) + 2He		
RC(	46	)=	10^-19*(Tg/Te)^4.5	;	%	2e + H+ => H + e		
RC(	47	)=	1.86*10^-7/Te^0.43	;	%	e + H2+ => 2H		
RC(	48	)=	10^-19*(Tg/Te)^4.5	;	%	2e + H2+ => H2 + e		
RC(	49	)=	3.29*10^-9*(1.5*Te*kB/e)^0.578*exp(-7.56/(1.5*Te*kB/e))	;	%	e + H2 => 2H + e		
RC(	50	)=	4*10^-11*(1.5*Te*kB/e)^2.13*exp(-14.9/(1.5*Te*kB/e))	;	%	e + H2 => H2+ + 2e		
RC(	51	)=	3.12*10^-23/Te^1.5	;	%	e + N+ + M => N + M		
RC(	52	)=	10^-19*(Tg/Te)^4.5	;	%	2e + N+ => N + e		
RC(	53	)=	1.45*10^-11*(1.5*Te*kB/e)^2.58*exp(-8.54/(1.5*Te*kB/e))	;	%	e + N => N+ + 2e		
RC(	54	)=	2.8*10^-7*(300/Te)^0.5	;	%	e + N2+ => 2N		
if Te*kB/e > 3.3
    RC(	55	)=	0.8765807+2.8182/(1+(Te*kB/e/5.78903)^2.997126)*((2*Te*kB/me)^0.5)*100;	%	e + N2+ => N2+(B2Sigmauv0) + e		
else
    RC(55) = 0;
end
RC(	56	)=	3.12*10^-23/Te^1.5	;	%	e + N2+ + M => N2 + M		
RC(	57	)=	10^-19*(Tg/Te)^4.5	;	%	2e + N2+ => N2 + e		
RC(	58	)=	P(25)	;	%	e + N2 => N2+ + 2e		
RC(	59	)=	P(22)/4	;	%	e + N2 => N2(B3PIgv0) + e	*exp(-(Te-89785.507)^2/(Tg-300)^5)	
RC(	60	)=	P(22)/4	;	%	e + N2 => N2(B3PIgv1) + e	*exp(-(Te-89785.507)^2/(Tg-300)^5)	
RC(	61	)=	P(22)/4	;	%	e + N2 => N2(B3PIgv2) + e	*exp(-(Te-89785.507)^2/(Tg-300)^5)	
RC(	62	)=	P(22)/4	;	%	e + N2 => N2(B3PIgv3) + e	*exp(-(Te-89785.507)^2/(Tg-300)^5)	
RC(	63	)=	P(23)	;	%	e + N2 => N2(B3PIgv4) + e	*exp(-(Te-97982.6087)^2/(Tg-300)^5)	
RC(	64	)=	P(24)/3	;	%	e + N2 => N2(C3PIuv0) + e	*exp(-(Te-129715.942)^2/(Tg-300)^5)	
RC(	65	)=	P(24)/3	;	%	e + N2 => N2(C3PIuv1) + e	*exp(-(Te-129715.942)^2/(Tg-300)^5)	
RC(	66	)=	P(24)/3	;	%	e + N2 => N2(C3PIuv2) + e	*exp(-(Te-129715.942)^2/(Tg-300)^5)	
RC(	67	)=	10^7*exp(-75.42-1.44*(Te*kB/e)*log(Te*kB/e)+9*((Te*kB/e)^0.5)*log(Te*kB/e))*((2*Te*kB/me)^0.5)*100;	%	e + O => O + e + h?(777.4nm)		
RC(	68	)=	3.12*10^-23/Te^1.5	;	%	e + O+ + M => O + M		
RC(	69	)=	10^-19*(Tg/Te)^4.5	;	%	2e + O+ => O + e		
RC(	70	)=	4.75*10^-9*(1.5*Te*kB/e)^0.61*exp(-22.1/(1.5*Te*kB/e))	;	%	e + O => O+ + 2e		
RC(	71	)=	10^-31	;	%	e + O + O2 => O- + O2		
RC(	72	)=	10^-31	;	%	e + O + O2 => O + O2-		
RC(	73	)=	2*10^-7*(300/Te)	;	%	e + O2+ => 2O		
RC(	74	)=	10^-19*(Tg/Te)^4.5	;	%	2e + O2+ => O2 + e		
RC(	75	)=	3.12*10^-23/Te^1.5	;	%	e + O2+ + M => O2 + M		
RC(	76	)=	P(20)	;	%	e + O2 => O- + O		
RC(	77	)=	P(21)	;	%	e + O2 => O2+ + 2e		
RC(	78	)=	1.4*10^-29*Tg/Te*exp(-600/Tg)*exp(700*(Te^-Tg)/(Te*Tg))	;	%	e + 2O2 => O2- + O2		
RC(	79	)=	1.4*10^-31*(Tg/Te)^2*exp(-70/Tg)*exp(1500*(Te^-Tg)/(Te*Tg))	;	%	e + O2 + N2 => O2- + N2		
RC(	80	)=	10^-9	;	%	e + O3 => O2- + O		
RC(	81	)=	10^-11	;	%	e + O3 => O- + O2		
RC(	82	)=	1.78*10^-6*(1.5*Te*kB/e)^-0.614*exp(-11.5/(1.5*Te*kB/e))	;	%	e + O3 => O + O2 + e		
RC(	83	)=	10^-31	;	%	e + O3 + M => O3- + M		
RC(	84	)=	1.37*10^-6/Te^0.5	;	%	e + H2O+ => O + 2H		
RC(	85	)=	1.37*10^-6/Te^0.5	;	%	e + H2O+ => O + H2		
RC(	86	)=	2.73*10^-6/Te^0.5	;	%	e + H2O+ => OH + H		
RC(	87	)=	10^-19*(Tg/Te)^4.5	;	%	2e + H2O+ => H2O + e		
RC(	88	)=	9.65*10^-18*(1.5*Te*kB/e)^2.53*exp(-8.99/(1.5*Te*kB/e)) 	;	%	e + H2O => H2O+ + 2e		
RC(	89	)=	2.97*10^-15*(1.5*Te*kB/e)^-1.56*exp(-13.67/(1.5*Te*kB/e))	;	%	e + H2O => H2 + O-		
RC(	90	)=	4.42*10^-14*(1.5*Te*kB/e)^-2.0*exp(-13.39/(1.5*Te*kB/e))	;	%	e + H2O => OH + H-		
RC(	91	)=	9.89*10^-6*(1.5*Te*kB/e)^-1.64*exp(-67.6/(1.5*Te*kB/e))	;	%	e + H2O => OH+ + H + 2e		
RC(	92	)=	7.45*10^-9*(1.5*Te*kB/e)^0.34*exp(-54.2/(1.5*Te*kB/e))	;	%	e + H2O => H+ + OH + 2e		
RC(	93	)=	7.4*10^-10*(1.5*Te*kB/e)^0.45*exp(-55.5/(1.5*Te*kB/e))	;	%	e + H2O => O+ + H2 + 2e		
RC(	94	)=	8.49*10^-9*(1.5*Te*kB/e)^-1.23*exp(-74/(1.5*Te*kB/e))	;	%	e + H2O => H2+ + O + 2e		
RC(	95	)=	5.15*10^-9*(1.5*Te*kB/e)^0.62*exp(-10.9/(1.5*Te*kB/e))	;	%	e + H2O => OH + H + e		
RC(	96	)=	9.6*10^-10*(1.5*Te*kB/e)^-1.7*exp(-13.31/(1.5*Te*kB/e))	;	%	e + H2O => OH- + H		
RC(	97	)=	1.17*10^(-8)*Te^0.52*exp(-44642.8/Te); %	e + OH => OH(A2Sigmav0) + e		
RC(	98	)=	10^-19*(Tg/Te)^4.5	;	%	2e + OH+ => OH + e		
RC(	99	)=	1.07*10^-5/Te^0.85	;	%	e + NO+ => N + O		
RC(	100	)=	3.12*10^-23/Te^1.5	;	%	e + NO+ + M => NO + M		
RC(	101	)=	10^-19*(Tg/Te)^4.5	;	%	2e + NO+ => NO + e		
RC(	102	)=	8*10^-31	;	%	e + NO + M => NO- + M		
RC(	103	)=	3.46*10^-6/Te^0.5	;	%	e + NO2+ => NO + O		
RC(	104	)=	1.5*10^-30	;	%	e + NO2 + M => NO2- + M		
RC(	105	)=	10^-11	;	%	e + NO2 => O- + NO		
RC(	106	)=	3.46*10^-6/Te^0.5	;	%	e + N2O+ => N2 + O		
RC(	107	)=	2*10^-10	;	%	e + N2O => O- + N2		
RC(	108	)=	5*10^-8	;	%	e + HNO3 => NO2- + OH		
RC(	109	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + H- => 2H		
RC(	110	)=	3.8*10^-10	;	%	H+ + O => O+ + H
RC(	111	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + O- => O + H
RC(	112	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + OH- => OH + H
RC(	113	)=	8.2*10^-9	;	%	H+ + H2O => H2O+ + H
RC(	114	)=	1.9*10^-9	;	%	H+ + NO => NO+ + H
RC(	115	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + NO- => NO + H
RC(	116	)=	1.17*10^-9	;	%	H+ + O2 => O2+ + H
RC(	117	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + O2- => O2 + H
RC(	118	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + N2O- => N2O + H
RC(	119	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + NO2- => NO2 + H
RC(	120	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + O3- => O3 + H
RC(	121	)=	2*10^-7*(300/Tg)^0.5	;	%	H+ + NO3- => NO3 + H
RC(	122	)=	1.8*10^-9	;	%	H + H- => H2 + e
RC(	123	)=	6.4*10^-10	;	%	H + H2+ => H+ + H2
RC(	124	)=	6.8*10^-10	;	%	H + O+ => H+ + O
RC(	125	)=	1.62*10^-32	;	%	H + O + M => OH + M
RC(	126	)=	6.1*10^-26/Tg^2	;	%	H + OH + M => H2O + M
RC(	127	)=	1.8*10^-9	;	%	H + OH- => H2O + e
RC(	128	)=	10^-32*exp(300/Tg)	;	%	H + NO + M => HNO + M
RC(	129	)=	5.4*10^-32*(Tg/300)^-1.8	;	%	H + O2 + M => HO2 + M
RC(	130	)=	1.4*10^-9	;	%	H + O2- => HO2 + e
RC(	131	)=	1.8*10^-30/Tg	;	%	2H + M => H2 + M
RC(	132	)=	3*10^-11*exp(-500/Tg)	;	%	H + HNO => NO + H2
RC(	133	)=	5.6*10^-12	;	%	H + HO2 => H2 + O2
RC(	134	)=	2.4*10^-12	;	%	H + HO2 => O + H2O
RC(	135	)=	4.2*10^-10*exp(-950/Tg)	;	%	H + HO2 => 2OH
RC(	136	)=	1.69*10^-11*exp(-1800/Tg)	;	%	H + H2O2 => OH + H2O
RC(	137	)=	2.8*10^-12*exp(-1900/Tg)	;	%	H + H2O2 => HO2 + H2
RC(	138	)=	1.47*10^-10	;	%	H + NO2 => OH + NO
RC(	139	)=	4*10^-10	;	%	H + NO2- => OH- + NO
RC(	140	)=	2*10^-11*exp(-3700/Tg)	;	%	H + HNO2 => NO2 + H2
RC(	141	)=	2.8*10^-11*(Tg/300)^0.75	;	%	H + O3 => OH + O2		
RC(	142	)=	8.4*10^-10	;	%	H + O3- => OH- + O2		
RC(	143	)=	5.8*10^-10*exp(-750/Tg)	;	%	H + NO3 => OH + NO2		
RC(	144	)=	1.39*10^-14*(Tg/298)^3.29*exp(-3160/Tg)	;	%	H + HNO3 => NO2 + H2O		
RC(	145	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + H2+ => H + H2		
RC(	146	)=	10^-7	;	%	H- + H2+ => 3H		
RC(	147	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + N+ => N + H		
RC(	148	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + O+ => O + H		
RC(	149	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + OH+ => H + OH		
RC(	150	)=	10^-7	;	%	H- + OH+ => 2H + O		
RC(	151	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + NO+ => H + NO		
RC(	152	)=	10^-7	;	%	H- + NO+ => H + N + O		
RC(	153	)=	3.8*10^-9	;	%	H- + H2O => OH- + H2		
RC(	154	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + H2O+ => H + H2O		
RC(	155	)=	10^-7	;	%	H- + H2O+ => 2H + OH		
RC(	156	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + N2+ => H + N2		
RC(	157	)=	10^-7	;	%	H- + N2+ => H + 2N		
RC(	158	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + O2+ => H + O2		
RC(	159	)=	10^-7	;	%	H- + O2+ => H + 2O		
RC(	160	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + N2O+ => H + N2O		
RC(	161	)=	10^-7	;	%	H- + N2O+ => H + N2 + O		
RC(	162	)=	1.1*10^-9	;	%	H- + N2O => OH- + N2		
RC(	163	)=	2*10^-7*(300/Tg)^0.5	;	%	H- + NO2+ => H + NO2		
RC(	164	)=	10^-7	;	%	H- + NO2+ => H + N + O2		
RC(	165	)=	2.9*10^-9	;	%	H- + NO2 => NO2- + H		
RC(	166	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + O- => O + H2		
RC(	167	)=	10^-7	;	%	H2+ + O- => O + 2H		
RC(	168	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + OH- => OH + H2		
RC(	169	)=	10^-7	;	%	H2+ + OH- => OH + 2H		
RC(	170	)=	3.86*10^-9	;	%	H2+ + H2O => H2O+ + H2		
RC(	171	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + NO- => NO + H2		
RC(	172	)=	10^-7	;	%	H2+ + NO- => NO + 2H		
RC(	173	)=	7.83*10^-10	;	%	H2+ + O2 => O2+ + H2		
RC(	174	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + O2- => O2 + H2		
RC(	175	)=	10^-7	;	%	H2+ + O2- => O2 + 2H		
RC(	176	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + N2O- => N2O + H2		
RC(	177	)=	10^-7	;	%	H2+ + N2O- => N2O + 2H		
RC(	178	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + NO2- => NO2 + H2		
RC(	179	)=	10^-7	;	%	H2+ + NO2- => NO2 + 2H		
RC(	180	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + O3- => O3 + H2		
RC(	181	)=	10^-7	;	%	H2+ + O3- => O3 + 2H		
RC(	182	)=	2*10^-7*(300/Tg)^0.5	;	%	H2+ + NO3- => NO3 + H2		
RC(	183	)=	10^-7	;	%	H2+ + NO3- => NO3 + 2H		
RC(	184	)=	1.7*10^-9	;	%	H2 + O+ => OH+ + H		
RC(	185	)=	3.3*10^-11	;	%	H2 + O- => OH- + H		
RC(	186	)=	7*10^-10	;	%	H2 + O- => H2O + e		
RC(	187	)=	9.7*10^-10	;	%	H2 + OH+ => H2O+ + H		
RC(	188	)=	3.2*10^-11*exp(-2600/Tg)	;	%	H2 + OH => H + H2O		
RC(	189	)=	3.6*10^-12	;	%	He + He(31S) => He2+ + e		
RC(	190	)=	1.11*10^-10	;	%	He + He(31P) => He2+ + e		
RC(	191	)=	7.36*10^-10	;	%	He + He(31D) => He2+ + e		
RC(	192	)=	3.625*10^-13	;	%	He + He(33S) => He2+ + e		
RC(	193	)=	5.7263*10^-11	;	%	He + He(33P) => He2+ + e		
RC(	194	)=	1.62*10^-10	;	%	He + He(33D) => He2+ + e		
RC(	195	)=	7.6*10^-10	;	%	He + He(43S) => He2+ + e		
RC(	196	)=	6.8*10^-10	;	%	He + He(41S) => He2+ + e		
RC(	197	)=	8.7*10^-10	;	%	He + He(43P) => He2+ + e		
RC(	198	)=	9.75*10^-10	;	%	He + He(43D) => He2+ + e		
RC(	199	)=	1.626*10^-9	;	%	He + He(41D) => He2+ + e		
RC(	200	)=	9.1973*10^-10	;	%	He + He(43F) => He2+ + e		
RC(	201	)=	9.26*10^-10	;	%	He + He(41F) => He2+ + e		
RC(	202	)=	4.7*10^-10	;	%	He + He(41P) => He2+ + e		
RC(	203	)=	6.7*10^-11	;	%	He + He(33D) => He(33P) + He		
RC(	204	)=	5.8*10^-10	;	%	He + He(31P) => He(31D) + He		
RC(	205	)=	10^-10	;	%	He + He(43P) => He(43S) + He		
RC(	206	)=	9.3*10^-11	;	%	He + He(43D) => He(43P) + He		
RC(	207	)=	5.8*10^-11	;	%	He + He(43D) => He(43S) + He		
RC(	208	)=	2.2*10^-10	;	%	He + He(41D) => He(41S) + He		
RC(	209	)=	1.8*10^-10	;	%	He + He(43F) => He(43D) + He		
RC(	210	)=	6.6*10^-11	;	%	He + He(43F) => He(43P) + He		
RC(	211	)=	4.1*10^-11	;	%	He + He(43F) => He(43S) + He		
RC(	212	)=	5.9*10^-10	;	%	He + He(41F) => He(41D) + He		
RC(	213	)=	1.6*10^-10	;	%	He + He(41F) => He(41S) + He		
RC(	214	)=	7.6*10^-10	;	%	He + He(41P) => He(41F) + He		
RC(	215	)=	7.4*10^-10	;	%	He + He(41P) => He(41D) + He		
RC(	216	)=	3.6*10^-10	;	%	He + He(41P) => He(41S) + He		
RC(	217	)=	8.1*10^-11	;	%	He + He(33P) => He(33D) + He		
RC(	218	)=	3.3*10^-10	;	%	He + He(31D) => He(31P) + He		
RC(	219	)=	1.7*10^-10	;	%	He + He(43S) => He(43P) + He		
RC(	220	)=	1.4*10^-10	;	%	He + He(43P) => He(43D) + He		
RC(	221	)=	1.5*10^-10	;	%	He + He(43S) => He(43D) + He		
RC(	222	)=	8.2*10^-10	;	%	He + He(41S) => He(41D) + He		
RC(	223	)=	2.5*10^-10	;	%	He + He(43D) => He(43F) + He		
RC(	224	)=	1.3*10^-10	;	%	He + He(43P) => He(43F) + He		
RC(	225	)=	1.5*10^-10	;	%	He + He(43S) => He(43F) + He		
RC(	226	)=	8.3*10^-10	;	%	He + He(41D) => He(41F) + He		
RC(	227	)=	8.3*10^-11	;	%	He + He(41S) => He(41F) + He		
RC(	228	)=	3.2*10^-10	;	%	He + He(41F) => He(41P) + He		
RC(	229	)=	4.3*10^-10	;	%	He + He(41D) => He(41P) + He		
RC(	230	)=	7.8*10^-10	;	%	He + He(41S) => He(41P) + He		
RC(	231	)=	1.8*10^-31	;	%	2He + He(21P) => 3He		
RC(	232	)=	1.3*10^-33	;	%	2He + He(21S) => 3He		
RC(	233	)=	1.4*10^-31*(Tg/300)^-0.6	;	%	2He + He+ => He2+ + He		
RC(	234	)=	1.4*10^-6*Tg^-0.67*exp(-28100/Tg)	;	%	He + He2+ => 2He + He+		
RC(	235	)=	1.022*10^7	;	%	He(23P) => He(23S) + hv(1083nm)		
RC(	236	)=	1.976*10^6	;	%	He(21P) => He(21S) + hv(2058.7nm)		
RC(	237	)=	1.799*10^9	;	%	He(21P) => He + hv(58.43nm)		
RC(	238	)=	1.81*10^7	;	%	He(31S) => He(21P) + hv(728.14nm)		
RC(	239	)=	1.338*10^7	;	%	He(31P) => He(21S) + hv(501.57nm)		
RC(	240	)=	5.66*10^8	;	%	He(31P) => He + hv(53.7nm)		
RC(	241	)=	6.38*10^7	;	%	He(31D) => He(21P) + hv(667.82nm)		
RC(	242	)=	1.54*10^7	;	%	He(33S) => He(23P) + hv(706.52nm)		
RC(	243	)=	9.478*10^6	;	%	He(33P) => He(23S) + hv(388.86nm)		
RC(	244	)=	7.06*10^7	;	%	He(33D) => He(23P) + hv(587.56nm)		
RC(	245	)=	6.55*10^6	;	%	He(41S) => He(21P) + hv(504.77nm)		
RC(	246	)=	4.59*10^6	;	%	He(41S) => He(31P) + hv(211.37nm)		
RC(	247	)=	7.17*10^6	;	%	He(41P) => He(21S) + hv(396.47nm)		
RC(	248	)=	2.46*10^8	;	%	He(41P) => He + hv(52.22nm)		
RC(	249	)=	2.02*10^7	;	%	He(41D) => He(21P) + hv(492.19nm)		
RC(	250	)=	7.11*10^6	;	%	He(41D) => He(31P) + hv(190.89nm)		
RC(	251	)=	1.38*10^7	;	%	He(41F) => He(31D) + hv(186.97nm)		
RC(	252	)=	5.89*10^6	;	%	He(43S) => He(23P) + hv(471.32nm)		
RC(	253	)=	3.62*10^6	;	%	He(43S) => He(33P) + hv(211.25nm)		
RC(	254	)=	5.05*10^6	;	%	He(43P) => He(23S) + hv(318.77nm)		
RC(	255	)=	2.51*10^7	;	%	He(43D) => He(23P) + hv(447.15nm)		
RC(	256	)=	6.68*10^6	;	%	He(43D) => He(33P) + hv(170.02nm)		
RC(	257	)=	1.55*10^6	;	%	He(43F) => He(33D) + hv(186.85nm)		
RC(	258	)=	10^-12	;	%	N+ + O => O+ + N		
RC(	259	)=	10^-29	;	%	N+ + O + M => NO+ + M		
RC(	260	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + O- => O + N		
RC(	261	)=	3.4*10^-10	;	%	N+ + OH => OH+ + N		
RC(	262	)=	3.4*10^-10	;	%	N+ + OH => NO+ + H		
RC(	263	)=	10^-29	;	%	N+ + N + M => N2+ + M		
RC(	264	)=	1.19*10^-9	;	%	N+ + H2O => H2O+ + N		
RC(	265	)=	2.1*10^-10	;	%	N+ + H2O => NO+ + H2		
RC(	266	)=	4.72*10^-10	;	%	N+ + NO => NO+ + N		
RC(	267	)=	8.33*10^-11	;	%	N+ + NO => N2+ + O		
RC(	268	)=	10^-12	;	%	N+ + NO => O+ + N2		
RC(	269	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + NO- => NO + N		
RC(	270	)=	2.7*10^-10	;	%	N+ + O2 => NO+ + O		
RC(	271	)=	2.8*10^-11	;	%	N+ + O2 => O+ + NO		
RC(	272	)=	3*10^-10	;	%	N+ + O2 => O2+ + N		
RC(	273	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + O2- => O2 + N		
RC(	274	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + O3- => O3 + N		
RC(	275	)=	5.5*10^-10	;	%	N+ + N2O => NO+ + N2		
RC(	276	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + N2O- => N2O + N		
RC(	277	)=	3*10^-10	;	%	N+ + NO2 => NO2+ + N		
RC(	278	)=	5*10^-10	;	%	N+ + NO2 => NO+ + NO		
RC(	279	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + NO2- => NO2 + N		
RC(	280	)=	5*10^-10	;	%	N+ + O3 => NO+ + O2		
RC(	281	)=	2*10^-7*(300/Tg)^0.5	;	%	N+ + NO3- => NO3 + N		
RC(	282	)=	10^-29	;	%	N + O+ + M => NO+ + M		
RC(	283	)=	6.3*10^-33*exp(140/Tg)	;	%	N + O + M => NO + M		
RC(	284	)=	2.6*10^-10	;	%	N + O- => NO + e		
RC(	285	)=	7.5*10^-11	;	%	N + OH => H + NO		
RC(	286	)=	1.9*10^-10	;	%	N + H2O+ => NO+ + H2		
RC(	287	)=	8.3*10^-34*exp(500/Tg)	;	%	2N + M => N2 + M		
RC(	288	)=	10^-12	;	%	N + N2+ => N+ + N2		
RC(	289	)=	10^-29*(300/Tg)	;	%	N + NO+ + M => N2O+ + M		
RC(	290	)=	2.1*10^-11*exp(100/Tg)	;	%	N + NO => N2 + O		
RC(	291	)=	1.5*10^-10	;	%	N + O2+ => NO+ + O		
RC(	292	)=	1.5*10^-11*exp(-3600/Tg)	;	%	N + O2 => NO + O		
RC(	293	)=	5*10^-10	;	%	N + O2- => NO2 + e		
RC(	294	)=	1.7*10^-11*exp(-1000/Tg)	;	%	N + HO2 => NO + OH		
RC(	295	)=	5.8*10^-12*exp(220/Tg)	;	%	N + NO2 => N2O + O		
RC(	296	)=	9.1*10^-13	;	%	N + NO2 => N2 + 2O		
RC(	297	)=	6*10^-13	;	%	N + NO2 => 2NO		
RC(	298	)=	7*10^-13	;	%	N + NO2 => N2 + O2		
RC(	299	)=	10^-12	;	%	N + NO2- => N2 + O2 + e		
RC(	300	)=	5*10^-16	;	%	N + O3 => NO + O2		
RC(	301	)=	10^-12	;	%	N + NO3- => N2 + O3 + e		
RC(	302	)=	10^-29	;	%	O+ + O + M => O2+ + M		
RC(	303	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + O- => 2O		
RC(	304	)=	3.6*10^-10	;	%	O+ + OH => O2+ + H		
RC(	305	)=	3.3*10^-10	;	%	O+ + OH => OH+ + O		
RC(	306	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + OH- => O + OH		
RC(	307	)=	10^-12	;	%	O+ + NO => NO+ + O		
RC(	308	)=	3*10^-12	;	%	O+ + NO => O2+ + N		
RC(	309	)=	3.2*10^-9	;	%	O+ + H2O => H2O+ + O		
RC(	310	)=	6*10^-29*(300/Tg)^2	;	%	O+ + N2 + M => NO+ + N + M		
RC(	311	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + NO- => NO + O		
RC(	312	)=	2.1*10^-11*(300/Tg)^0.5	;	%	O+ + O2 => O2+ + O		
RC(	313	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + O2- => O2 + O		
RC(	314	)=	6.3*10^-10	;	%	O+ + N2O => N2O+ + O		
RC(	315	)=	2.3 *10^-10	;	%	O+ + N2O => NO+ + NO		
RC(	316	)=	2*10^-11	;	%	O+ + N2O => O2+ + N2		
RC(	317	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + N2O- => N2O + O		
RC(	318	)=	5*10^-10	;	%	O+ + NO2 => NO+ + O2		
RC(	319	)=	1.6*10^-9	;	%	O+ + NO2 => NO2+ + O		
RC(	320	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + NO2- => NO2 + O		
RC(	321	)=	10^-10	;	%	O+ + O3 => O2+ + O2		
RC(	322	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + O3- => O3 + O		
RC(	323	)=	2*10^-7*(300/Tg)^0.5	;	%	O+ + NO3- => NO3 + O		
RC(	324	)=	1.4*10^-10	;	%	O + O- => O2 + e		
RC(	325	)=	2.2*10^-11*exp(-350/Tg)	;	%	O + OH => H + O2		
RC(	326	)=	2*10^-10	;	%	O + OH- => HO2 + e		
RC(	327	)=	5.5*10^-11	;	%	O + H2O+ => O2+ + H2		
RC(	328	)=	1*10^-11*(300/Tg)^0.5	;	%	O + N2+ => O+ + N2		
RC(	329	)=	1.4 *10^-10	;	%	O + N2+ => NO+ + N		
RC(	330	)=	10^-31*(300/Tg)^1.6	;	%	O + NO + M => NO2 + M		
RC(	331	)=	3*10^-10	;	%	O + NO- => O- + NO		
RC(	332	)=	5.99*10^-11	;	%	O + HNO => OH + NO		
RC(	333	)=	3.4*10^-34*(Tg/300)^-1.2	;	%	O + O2 + M => O3 + M		
RC(	334	)=	3.3*10^-10	;	%	O + O2- => O- + O2		
RC(	335	)=	1.5*10^-10	;	%	O + O2- => O3 + e		
RC(	336	)=	3.2*10^-35*exp(900/Tg)	;	%	2O + M => O2 + M		
RC(	337	)=	8.3*10^-11*exp(-500/Tg)	;	%	O + HO2 => OH + O2		
RC(	338	)=	3.3*10^-11*exp(-2950/Tg)	;	%	O + H2O2 => OH + HO2		
RC(	339	)=	6.5*10^-12*exp(120/Tg)	;	%	O + NO2 => NO + O2		
RC(	340	)=	9*10^-32*(300/Tg)^2	;	%	O + NO2 + M => NO3 + M		
RC(	341	)=	10^-12	;	%	O + NO2- => NO3 + e		
RC(	342	)=	2*10^-11*exp(-3000/Tg)	;	%	O + HNO2 => NO2 + OH		
RC(	343	)=	8*10^-12*exp(-2060/Tg)	;	%	O + O3 => 2O2		
RC(	344	)=	3*10^-10	;	%	O + O3- => 2O2 + e		
RC(	345	)=	10^-11	;	%	O + O3- => O2- + O2		
RC(	346	)=	1.7*10^-11	;	%	O + NO3 => O2 + NO2		
RC(	347	)=	10^-12	;	%	O + NO3- => NO2 + O2 + e		
RC(	348	)=	10^-7	;	%	O- + OH+ => 2O + H		
RC(	349	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + OH+ => O + OH		
RC(	350	)=	10^-12	;	%	O- + N2 => N2O + e		
RC(	351	)=	2.6*10^-10	;	%	O- + NO => NO2 + e		
RC(	352	)=	10^-29	;	%	O- + NO + M => NO2- + M		
RC(	353	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + N2+ => O + N2		
RC(	354	)=	10^-7	;	%	O- + N2+ => O + 2N		
RC(	355	)=	10^-7	;	%	O- + NO+ => 2O + N		
RC(	356	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + NO+ => O + NO		
RC(	357	)=	10^-7	;	%	O- + O2+ => 3O		
RC(	358	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + O2+ => O + O2		
RC(	359	)=	10^-12	;	%	O- + O2 => O3 + e		
RC(	360	)=	1.1*10^-30*(300/Tg)	;	%	O- + O2 + M => O3- + M		
RC(	361	)=	10^-7	;	%	O- + H2O+ => O + OH + H		
RC(	362	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + H2O+ => O + H2O		
RC(	363	)=	1.4*10^-9	;	%	O- + H2O => OH- + OH		
RC(	364	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + N2O+ => O + N2O		
RC(	365	)=	10^-7	;	%	O- + N2O+ => 2O + N2		
RC(	366	)=	2*10^-10	;	%	O- + N2O => NO- + NO		
RC(	367	)=	2*10^-12	;	%	O- + N2O => N2O- + O		
RC(	368	)=	2*10^-7*(300/Tg)^0.5	;	%	O- + NO2+ => O + NO2		
RC(	369	)=	10^-7	;	%	O- + NO2+ => O + N + O2		
RC(	370	)=	1.2*10^-9	;	%	O- + NO2 => NO2- + O		
RC(	371	)=	8*10^-10	;	%	O- + O3 => O3- + O		
RC(	372	)=	3*10^-10	;	%	O- + O3 => 2O2 + e		
RC(	373	)=	3*10^-10	;	%	O- + NO3 => NO3- + O		
RC(	374	)=	7*10^-10	;	%	OH+ + OH => H2O+ + O		
RC(	375	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + OH- => 2OH		
RC(	376	)=	10^-7	;	%	OH+ + OH- => OH + O + H		
RC(	377	)=	1.59*10^-9	;	%	OH+ + H2O => H2O+ + OH		
RC(	378	)=	5.2*10^-10	;	%	OH+ + NO => NO+ + OH		
RC(	379	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + NO- => NO + OH		
RC(	380	)=	10^-7	;	%	OH+ + NO- => NO + O + H		
RC(	381	)=	5.9*10^-10	;	%	OH+ + O2 => O2+ + OH		
RC(	382	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + O2- => O2 + OH		
RC(	383	)=	10^-7	;	%	OH+ + O2- => O2 + O + H		
RC(	384	)=	2.13*10^-10	;	%	OH+ + N2O => N2O+ + OH		
RC(	385	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + N2O- => N2O + OH		
RC(	386	)=	10^-7	;	%	OH+ + N2O- => N2O + O + H		
RC(	387	)=	1.3*10^-9	;	%	OH+ + NO2 => NO+ + HO2		
RC(	388	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + NO2- => NO2 + OH		
RC(	389	)=	10^-7	;	%	OH+ + NO2- => NO2 + O + H		
RC(	390	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + O3- => O3 + OH		
RC(	391	)=	10^-7	;	%	OH+ + O3- => O3 + O + H		
RC(	392	)=	2*10^-7*(300/Tg)^0.5	;	%	OH+ + NO3- => NO3 + OH		
RC(	393	)=	10^-7	;	%	OH+ + NO3- => NO3 + O + H		
RC(	394	)=	3.48*10^4	;	%	OH(A2Sigmav0) => OH + h?(308.9nm)		
RC(	395	)=	8.8*10^-12*exp(-503/Tg)	;	%	2OH => O + H2O
RC(	396	)=	6.9*10^-31*(Tg/300)^-0.8	;	%	2OH + M => H2O2 + M
RC(	397	)=	7.4*10^-31*(300/Tg)^2.4	;	%	OH + NO + M => HNO2 + M
RC(	398	)=	8*10^-11*exp(-500/Tg)	;	%	OH + HNO => NO + H2O
RC(	399	)=	4.8*10^-11*exp(250/Tg)	;	%	OH + HO2 => O2 + H2O
RC(	400	)=	2.9*10^-12*exp(-160/Tg)	;	%	OH + H2O2 => HO2 + H2O
RC(	401	)=	2.2*10^-30*(300/Tg)^2.9	;	%	OH + NO2 + M => HNO3 + M
RC(	402	)=	1.8*10^-11*exp(-390/Tg)	;	%	OH + HNO2 => NO2 + H2O
RC(	403	)=	1.6*10^-12*exp(-1000/Tg)	;	%	OH + O3 => HO2 + O2
RC(	404	)=	2*10^-11	;	%	OH + NO3 => HO2 + NO2
RC(	405	)=	1.5*10^-14*exp(650/Tg)	;	%	OH + HNO3 => NO3 + H2O
RC(	406	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + N+ => N + OH
RC(	407	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + H2O+ => OH + H2O
RC(	408	)=	10^-7	;	%	OH- + H2O+ => 2OH + H
RC(	409	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + N2+ => OH + N2
RC(	410	)=	10^-7	;	%	OH- + N2+ => OH + 2N
RC(	411	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + NO+ => OH + NO
RC(	412	)=	10^-7	;	%	OH- + NO+ => OH + N + O
RC(	413	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + O2+ => OH + O2
RC(	414	)=	10^-7	;	%	OH- + O2+ => OH + 2O
RC(	415	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + N2O+ => OH + N2O
RC(	416	)=	10^-7	;	%	OH- + N2O+ => OH + N2 + O
RC(	417	)=	2*10^-7*(300/Tg)^0.5	;	%	OH- + NO2+ => OH + NO2
RC(	418	)=	10^-7	;	%	OH- + NO2+ => OH + N + O2
RC(	419	)=	1.9*10^-9	;	%	OH- + NO2 => NO2- + OH
RC(	420	)=	9*10^-10	;	%	OH- + O3 => O3- + OH
RC(	421	)=	4.3*10^-10	;	%	H2O+ + O2 => O2+ + H2O
RC(	422	)=	2*10^-7*(300/Tg)^0.5	;	%	H2O+ + O2- => O2 + H2O
RC(	423	)=	10^-7	;	%	H2O+ + O2- => O2 + OH + H
RC(	424	)=	4.6*10^-10	;	%	H2O+ + NO => NO+ + H2O
RC(	425	)=	2*10^-7*(300/Tg)^0.5	;	%	H2O+ + NO- => NO + H2O
RC(	426	)=	10^-7	;	%	H2O+ + NO- => NO + OH + H
RC(	427	)=	2*10^-7*(300/Tg)^0.5	;	%	H2O+ + N2O- => N2O + H2O
RC(	428	)=	10^-7	;	%	H2O+ + N2O- => N2O + OH + H
RC(	429	)=	1.2*10^-9	;	%	H2O+ + NO2 => NO2+ + H2O
RC(	430	)=	2*10^-7*(300/Tg)^0.5	;	%	H2O+ + NO2- => NO2 + H2O
RC(	431	)=	10^-7	;	%	H2O+ + NO2- => NO2 + OH + H
RC(	432	)=	2*10^-7*(300/Tg)^0.5	;	%	H2O+ + O3- => O3 + H2O
RC(	433	)=	10^-7	;	%	H2O+ + O3- => O3 + OH + H
RC(	434	)=	2*10^-7*(300/Tg)^0.5	;	%	H2O+ + NO3- => NO3 + H2O
RC(	435	)=	10^-7	;	%	H2O+ + NO3- => NO3 + OH + H
RC(	436	)=	2.3*10^-9	;	%	H2O + N2+ => H2O+ + N2
RC(	437	)=	3.9*10^-10	;	%	N2+ + NO => NO+ + N2
RC(	438	)=	2*10^-7*(300/Tg)^0.5	;	%	N2+ + NO- => NO + N2
RC(	439	)=	10^-7	;	%	N2+ + NO- => NO + 2N
RC(	440	)=	5*10^-11	;	%	N2+ + O2 => O2+ + N2
RC(	441	)=	2*10^-7*(300/Tg)^0.5	;	%	N2+ + O2- => O2 + N2
RC(	442	)=	10^-7	;	%	N2+ + O2- => O2 + 2N
RC(	443	)=	6*10^-10	;	%	N2+ + N2O => N2O+ + N2
RC(	444	)=	4*10^-10	;	%	N2+ + N2O => NO+ + N + N2
RC(	445	)=	2*10^-7*(300/Tg)^0.5	;	%	N2+ + N2O- => N2O + N2
RC(	446	)=	10^-7	;	%	N2+ + N2O- => N2O + 2N
RC(	447	)=	5*10^-11	;	%	N2+ + NO2 => NO+ + N2O
RC(	448	)=	3*10^-10	;	%	N2+ + NO2 => NO2+ + N2
RC(	449	)=	2*10^-7*(300/Tg)^0.5	;	%	N2+ + NO2- => NO2 + N2
RC(	450	)=	10^-7	;	%	N2+ + NO2- => NO2 + 2N
RC(	451	)=	2*10^-7*(300/Tg)^0.5	;	%	N2+ + O3- => O3 + N2
RC(	452	)=	10^-7	;	%	N2+ + O3- => O3 + 2N
RC(	453	)=	10^-10	;	%	N2+ + O3 => O2+ + O + N2
RC(	454	)=	2*10^-7*(300/Tg)^0.5	;	%	N2+ + NO3- => NO3 + N2
RC(	455	)=	10^-7	;	%	N2+ + NO3- => NO3 + 2N
RC(	456	)=	6.53*10^8	;	%	N2+(B2Sigmauv0) => N2+ + h?(391.44nm)
RC(	457	)=	6.34*10^8	;	%	N2+(B2Sigmauv0) => N2+(X2Sigmagv1) + h?(427.81nm)
RC(	458	)=	10^-17	;	%	N2 + O2+ => NO+ + NO
RC(	459	)=	1.9*10^-12*(Tg/300)^0.5*exp(-4990/Tg)	;	%	N2 + O2- => N2 + O2 + e
RC(	460	)=	4.2*10^-9	;	%	N2+(B2Sigmauv0) + N2 => N2+ + N2
RC(	461	)=	12.1*10^6	;	%	N2(C3PIuv0) => N2(B3PIgv0) + hv(337.13nm)
RC(	462	)=	8.6*10^6	;	%	N2(C3PIuv0) => N2(B3PIgv1) + hv(357.69nm)
RC(	463	)=	3*10^6	;	%	N2(C3PIuv0) => N2(B3PIgv2) + hv(380.49nm)
RC(	464	)=	10^6	;	%	N2(C3PIuv0) => N2(B3PIgv3) + hv(405.94nm)
RC(	465	)=	11*10^6	;	%	N2(C3PIuv1) => N2(B3PIgv0) + hv(315.93nm)
RC(	466	)=	5.2*10^6	;	%	N2(C3PIuv1) => N2(B3PIgv2) + hv(353.67nm)
RC(	467	)=	4.6*10^6	;	%	N2(C3PIuv1) => N2(B3PIgv3) + hv(375.54nm)
RC(	468	)=	2.4*10^6	;	%	N2(C3PIuv1) => N2(B3PIgv4) + hv(399.85nm)
RC(	469	)=	3.12*10^6	;	%	N2(C3PIuv2) => N2(B3PIgv4) + hv(371.05nm)
RC(	470	)=	2*10^-7*(300/Tg)^0.5	;	%	NO+ + NO- => 2NO
RC(	471	)=	10^-7	;	%	NO+ + NO- => NO + N + O
RC(	472	)=	10^-7	;	%	NO+ + O2- => O2 + N + O
RC(	473	)=	2*10^-7*(300/Tg)^0.5	;	%	NO+ + O2- => O2 + NO
RC(	474	)=	2*10^-7*(300/Tg)^0.5	;	%	NO+ + N2O- => N2O + NO
RC(	475	)=	10^-7	;	%	NO+ + N2O- => N2O + N + O
RC(	476	)=	2*10^-7*(300/Tg)^0.5	;	%	NO+ + NO2- => NO2 + NO
RC(	477	)=	10^-7	;	%	NO+ + NO2- => NO2 + N + O
RC(	478	)=	10^-15	;	%	NO+ + O3 => NO2+ + O2
RC(	479	)=	10^-7	;	%	NO+ + O3- => O3 + N + O
RC(	480	)=	2*10^-7*(300/Tg)^0.5	;	%	NO+ + O3- => O3 + NO
RC(	481	)=	2*10^-7*(300/Tg)^0.5	;	%	NO+ + NO3- => NO3 + NO
RC(	482	)=	10^-7	;	%	NO+ + NO3- => NO3 + N + O
RC(	483	)=	5.9*10^-10	;	%	NO+ + N2O5 => NO2+ + 2NO2
RC(	484	)=	5*10^-12	;	%	NO + NO- => 2NO + e
RC(	485	)=	4.6*10^-10	;	%	NO + O2+ => NO+ + O2
RC(	486	)=	3.4*10^-12*exp(270/Tg)	;	%	NO + HO2 => OH + NO2
RC(	487	)=	3.3*10^-13*exp(-1000/Tg)	;	%	NO + HO2 => O2 + HNO
RC(	488	)=	2.3*10^-10	;	%	NO + N2O+ => NO+ + N2O
RC(	489	)=	2.75*10^-10	;	%	NO + NO2+ => NO+ + NO2
RC(	490	)=	2.75*10^-10	;	%	NO + NO2- => NO- + NO2
RC(	491	)=	1.8*10^-12*exp(-1370/Tg)	;	%	NO + O3 => NO2 + O2
RC(	492	)=	10^-11	;	%	NO + O3- => NO2- + O2
RC(	493	)=	10^-11	;	%	NO + O3- => NO3- + O
RC(	494	)=	1.8*10^-11*exp(110/Tg)	;	%	NO + NO3 => 2NO2
RC(	495	)=	3*10^-15	;	%	NO + NO3- => NO2- + NO2
RC(	496	)=	2.3*10^-13	;	%	NO- + H2 => NO + H2 + e
RC(	497	)=	2.4*10^-13	;	%	NO- + M => NO + M + e
RC(	498	)=	2*10^-7*(300/Tg)^0.5	;	%	NO- + O2+ => NO + O2
RC(	499	)=	10^-7	;	%	NO- + O2+ => NO + 2O
RC(	500	)=	5*10^-10	;	%	NO- + O2 => O2- + NO
RC(	501	)=	2*10^-7*(300/Tg)^0.5	;	%	NO- + NO2+ => NO + NO2
RC(	502	)=	10^-7	;	%	NO- + NO2+ => NO + N + O2
RC(	503	)=	3*10^-10	;	%	NO- + O3 => O3- + NO
RC(	504	)=	2*10^-7*(300/Tg)^0.5	;	%	NO- + N2O+ => NO + N2O
RC(	505	)=	10^-7	;	%	NO- + N2O+ => NO + N2 + O
RC(	506	)=	5.1*10^-12	;	%	NO- + N2O => NO + N2O + e
RC(	507	)=	2.8*10^-14	;	%	NO- + N2O => NO2- + N2
RC(	508	)=	3*10^-10	;	%	NO- + NO2 => NO2- + NO
RC(	509	)=	3*10^-10	;	%	NO- + NO3 => NO3- + NO
RC(	510	)=	2*10^-7*(300/Tg)^0.5	;	%	O2+ + O2- => 2O2
RC(	511	)=	10^-7	;	%	O2+ + O2- => O2 + 2O
RC(	512	)=	2*10^-7*(300/Tg)^0.5	;	%	O2+ + N2O- => N2O + O2
RC(	513	)=	10^-7	;	%	O2+ + N2O- => N2O + 2O
RC(	514	)=	6.6*10^-10	;	%	O2+ + NO2 => NO2+ + O2
RC(	515	)=	10^-11	;	%	O2+ + NO2 => NO+ + O3
RC(	516	)=	2*10^-7*(300/Tg)^0.5	;	%	O2+ + NO2- => NO2 + O2
RC(	517	)=	10^-7	;	%	O2+ + NO2- => NO2 + 2O
RC(	518	)=	2*10^-7*(300/Tg)^0.5	;	%	O2+ + O3- => O3 + O2
RC(	519	)=	10^-7	;	%	O2+ + O3- => O3 + 2O
RC(	520	)=	2*10^-7*(300/Tg)^0.5	;	%	O2+ + NO3- => NO3 + O2
RC(	521	)=	10^-7	;	%	O2+ + NO3- => NO3 + 2O
RC(	522	)=	8.8*10^-10	;	%	O2+ + N2O5 => NO2+ + NO3 + O2
RC(	523	)=	2.7*10^-10*(Tg/300)^0.5*exp(-5590/Tg)	;	%	O2 + O2- => 2O2 + e
RC(	524	)=	4.59*10^-11	;	%	O2 + N2O+ => NO+ + NO2
RC(	525	)=	2.24*10^-10	;	%	O2 + N2O+ => O2+ + N2O
RC(	526	)=	5.25*10^-12*exp(-1510/Tg)	;	%	O2 + HNO => NO + HO2
RC(	527	)=	1.66*10^-15	;	%	O2 + HNO => NO2 + OH
RC(	528	)=	2.3*10^-11	;	%	O2 + O3- => O3 + O2 + e
RC(	529	)=	1.4*10^-15*exp(-1600/Tg)	;	%	2HNO => N2O + H2O
RC(	530	)=	6*10^-10	;	%	O2- + O3 => O3 + O2 + e
RC(	531	)=	3.5*10^-10	;	%	O2- + O3 => O3- + O2
RC(	532	)=	2*10^-7*(300/Tg)^0.5	;	%	O2- + N2O+ => O2 + N2O
RC(	533	)=	10^-7	;	%	O2- + N2O+ => O2 + N2 + O
RC(	534	)=	10^-11	;	%	O2- + N2O => O3- + N2
RC(	535	)=	2*10^-7*(300/Tg)^0.5	;	%	O2- + NO2+ => O2 + NO2
RC(	536	)=	10^-7	;	%	O2- + NO2+ => O2 + N + O2
RC(	537	)=	7*10^-10	;	%	O2- + NO2 => NO2- + O2
RC(	538	)=	5*10^-10	;	%	O2- + NO3 => NO3- + O2
RC(	539	)=	2.8*10^-10	;	%	O2- + HNO3 => NO3- + HO2
RC(	540	)=	3.92*10^-10*exp(-11400/Tg)	;	%	M + O3 => O + O2 + M
RC(	541	)=	2.2*10^-13*exp(600/Tg)	;	%	2HO2 => H2O2 + O2
RC(	542	)=	1.4*10^-14*exp(-600/Tg)	;	%	HO2 + O3 => OH + 2O2
RC(	543	)=	4.8*10^-12	;	%	HO2 + NO3 => NO2 + OH + O2
RC(	544	)=	9.2*10^-13	;	%	HO2 + NO3 => HNO3 + O2
RC(	545	)=	1.2*10^-11	;	%	N2O+ + N2O => NO+ + NO + N2
RC(	546	)=	2*10^-7*(300/Tg)^0.5	;	%	N2O+ + N2O- => 2N2O
RC(	547	)=	10^-7	;	%	N2O+ + N2O- => N2O + N2 + O
RC(	548	)=	4.29*10^-10	;	%	N2O+ + NO2 => NO+ + N2 + O2
RC(	549	)=	2.21*10^-10	;	%	N2O+ + NO2 => NO2+ + N2O
RC(	550	)=	2*10^-7*(300/Tg)^0.5	;	%	N2O+ + NO2- => NO2 + N2O
RC(	551	)=	10^-7	;	%	N2O+ + NO2- => NO2 + N2 + O
RC(	552	)=	2*10^-7*(300/Tg)^0.5	;	%	N2O+ + O3- => O3 + N2O
RC(	553	)=	10^-7	;	%	N2O+ + O3- => O3 + N2 + O
RC(	554	)=	2*10^-7*(300/Tg)^0.5	;	%	N2O+ + NO3- => NO3 + N2O
RC(	555	)=	10^-7	;	%	N2O+ + NO3- => NO3 + N2 + O
RC(	556	)=	5*10^-13	;	%	N2O + NO2- => NO3- + N2
RC(	557	)=	2*10^-7*(300/Tg)^0.5	;	%	N2O- + NO2+ => N2O + NO2
RC(	558	)=	10^-7	;	%	N2O- + NO2+ => N2O + N + O2
RC(	559	)=	2*10^-7*(300/Tg)^0.5	;	%	NO2+ + NO2- => 2NO2
RC(	560	)=	10^-7	;	%	NO2+ + NO2- => NO2 + N + O2
RC(	561	)=	2*10^-7*(300/Tg)^0.5	;	%	NO2+ + O3- => O3 + NO2
RC(	562	)=	10^-7	;	%	NO2+ + O3- => O3 + N + O2
RC(	563	)=	2*10^-7*(300/Tg)^0.5	;	%	NO2+ + NO3- => NO3 + NO2
RC(	564	)=	10^-7	;	%	NO2+ + NO3- => NO3 + N + O2
RC(	565	)=	4*10^-12	;	%	NO2 + NO2- => NO3- + NO
RC(	566	)=	1.4*10^-13*exp(-2470/Tg)	;	%	NO2 + O3 => NO3 + O2
RC(	567	)=	7*10^-11	;	%	NO2 + O3- => NO2- + O3
RC(	568	)=	2*10^-11	;	%	NO2 + O3- => NO3- + O2
RC(	569	)=	2.3*10^-13*exp(-1600/Tg)	;	%	NO2 + NO3 => NO2 + NO + O2
RC(	570	)=	1.8*10^-11	;	%	NO2- + O3 => NO3- + O2
RC(	571	)=	5*10^-10	;	%	NO2- + NO3 => NO3- + NO2
RC(	572	)=	1.6*10^-9	;	%	NO2- + HNO3 => NO3- + HNO2
RC(	573	)=	7*10^-10	;	%	NO2- + N2O5 => NO3- + NO3 + NO
RC(	574	)=	3*10^-10	;	%	O3 + O3- => 3O2 + e
RC(	575	)=	5*10^-10	;	%	O3- + NO3 => NO3- + O3
RC(	576	)=	10^-20	;	%	2HNO2 => NO + NO2 + H2O
RC(	577	)=	1.6*10^-17	;	%	HNO2 + HNO3 => 2NO2 + H2O
RC(	578	)=	5*10^-12*exp(-3000/Tg)	;	%	2NO3 => 2NO2 + O2
RC(	579	) =	3.14887E-07	;		
RC(	580	) =	3.16744E-07	;		
RC(	581	) =	3.22677E-07	;		
RC(	582	) =	4.62656E-07	;		
RC(	583	) =	4.83768E-07	;		
RC(	584	) =	4.83918E-07	;		
RC(	585	) =	4.88836E-07	;		
RC(	586	) =	5.14375E-07	;		
RC(	587	) =	5.14526E-07	;		
RC(	588	) =	6.36868E-07	;		
RC(	589	) =	6.45687E-07	;		
RC(	590	) =	6.49249E-07	;		
RC(	591	) =	6.7228E-07	;		
RC(	592	) =	6.93028E-07	;		
RC(	593	) =	6.98071E-07	;		
RC(	594	) =	7.14829E-07	;		
RC(	595	) =	7.20462E-07	;		
RC(	596	) =	7.26671E-07	;		
RC(	597	) =	7.3717E-07	;		
RC(	598	) =	7.40407E-07	;		
RC(	599	) =	7.46716E-07	;		
RC(	600	) =	7.50956E-07	;		
RC(	601	) =	7.58596E-07	;		
RC(	602	) =	7.8603E-07	;		
RC(	603	) =	8.1029E-07	;		
RC(	604	) =	8.40609E-07	;		
RC(	605	) =	8.66788E-07	;		
RC(	606	) =	8.78555E-07	;		
RC(	607	) =	8.82569E-07	;		
RC(	608	) =	9.86421E-07	;		
RC(	609	) =	1.09433E-06	;		
RC(	610	) =	1.12949E-06	;		
RC(	611	) =	1.14393E-06	;		
RC(	612	) =	1.1703E-06	;		
RC(	613	) =	1.25538E-06	;		
RC(	614	) =	1.58843E-06	;		
RC(	615	) =	3.01915E-07	;		
RC(	616	) =	3.01618E-07	;		
RC(	617	) =	3.0067E-07	;		
RC(	618	) =	2.78293E-07	;		
RC(	619	) =	2.74918E-07	;		
RC(	620	) =	2.74894E-07	;		
RC(	621	) =	2.74108E-07	;		
RC(	622	) =	2.70025E-07	;		
RC(	623	) =	2.70001E-07	;		
RC(	624	) =	2.50444E-07	;		
RC(	625	) =	2.49034E-07	;		
RC(	626	) =	2.48465E-07	;		
RC(	627	) =	2.44783E-07	;		
RC(	628	) =	2.41466E-07	;		
RC(	629	) =	2.4066E-07	;		
RC(	630	) =	2.37981E-07	;		
RC(	631	) =	2.37081E-07	;		
RC(	632	) =	2.36088E-07	;		
RC(	633	) =	2.3441E-07	;		
RC(	634	) =	2.33892E-07	;		
RC(	635	) =	2.32884E-07	;		
RC(	636	) =	2.32206E-07	;		
RC(	637	) =	2.30985E-07	;		
RC(	638	) =	2.26599E-07	;		
RC(	639	) =	2.22721E-07	;		
RC(	640	) =	2.17874E-07	;		
RC(	641	) =	2.13689E-07	;		
RC(	642	) =	2.11808E-07	;		
RC(	643	) =	2.11167E-07	;		
RC(	644	) =	1.94565E-07	;		
RC(	645	) =	1.7888E-07	;		
RC(	646	) =	1.70608E-07	;		
RC(	647	) =	1.65334E-07	;		
RC(	648	) =	1.45208E-07	;		
RC(	649	) =	0	;		
RC(	650	) =	0	;		
RC(	651	) =	2.22139E-07	;		
RC(	652	) =	2.2159E-07	;		
RC(	653	) =	2.19836E-07	;		
RC(	654	) =	1.78443E-07	;		
RC(	655	) =	1.722E-07	;		
RC(	656	) =	1.72155E-07	;		
RC(	657	) =	1.70701E-07	;		
RC(	658	) =	1.63149E-07	;		
RC(	659	) =	1.63104E-07	;		
RC(	660	) =	1.26927E-07	;		
RC(	661	) =	1.24319E-07	;		
RC(	662	) =	1.23265E-07	;		
RC(	663	) =	1.16455E-07	;		
RC(	664	) =	1.1032E-07	;		
RC(	665	) =	1.08021E-07	;		
RC(	666	) =	9.98588E-08	;		
RC(	667	) =	9.71156E-08	;		
RC(	668	) =	9.40914E-08	;		
RC(	669	) =	8.89778E-08	;		
RC(	670	) =	8.74016E-08	;		
RC(	671	) =	8.43285E-08	;		
RC(	672	) =	8.22635E-08	;		
RC(	673	) =	7.85429E-08	;		
RC(	674	) =	7.14005E-08	;		
RC(	675	) =	6.70683E-08	;		
RC(	676	) =	6.16542E-08	;		
RC(	677	) =	5.62335E-08	;		
RC(	678	) =	5.3627E-08	;		
RC(	679	) =	5.29136E-08	;		
RC(	680	) =	4.24556E-08	;		
RC(	681	) =	3.45262E-08	;		
RC(	682	) =	3.33666E-08	;		
RC(	683	) =	3.29572E-08	;		
RC(	684	) =	2.74849E-08	;		
RC(	685	) =	5.64879E-09	;		
RC(	686	) =	0	;		
RC(	687	) =	0	;		
RC(	688	) =	0	;		
RC(	689	) =	0	;		
RC(	690	) =	0	;		
RC(	691	) =	0	;		
RC(	692	) =	0	;		
RC(	693	) =	0	;		
RC(	694	) =	0	;		
RC(	695	) =	0	;		
RC(	696	) =	1.2116E-08	;		
RC(	697	) =	2.01234E-08	;		
RC(	698	) =	2.33582E-08	;		
RC(	699	) =	4.42709E-08	;		
RC(	700	) =	6.31105E-08	;		
RC(	701	) =	6.65576E-08	;		
RC(	702	) =	7.6965E-08	;		
RC(	703	) =	8.04628E-08	;		
RC(	704	) =	8.43188E-08	;		
RC(	705	) =	9.08391E-08	;		
RC(	706	) =	9.28489E-08	;		
RC(	707	) =	9.67673E-08	;		
RC(	708	) =	9.94003E-08	;		
RC(	709	) =	1.08188E-07	;		
RC(	710	) =	2.127E-07	;		
RC(	711	) =	2.20974E-07	;		
RC(	712	) =	2.52322E-07	;		
RC(	713	) =	1.99259E-07	;		
RC(	714	) =	2.03188E-07	;		
RC(	715	) =	1.81111E-07	;		
RC(	716	) =	1.44792E-08	;		
RC(	717	) =	0	;		
RC(	718	) =	0	;		
RC(	719	) =	0	;		
RC(	720	) =	0	;		
RC(	721	) =	0	;		
RC(	722	) =	0	;		
RC(	723	) =	1.36178E-07	;		
RC(	724	) =	1.36747E-07	;		
RC(	725	) =	1.38563E-07	;		
RC(	726	) =	1.81413E-07	;		
RC(	727	) =	1.87875E-07	;		
RC(	728	) =	1.87921E-07	;		
RC(	729	) =	1.89427E-07	;		
RC(	730	) =	1.97245E-07	;		
RC(	731	) =	1.97291E-07	;		
RC(	732	) =	2.34741E-07	;		
RC(	733	) =	2.37441E-07	;		
RC(	734	) =	2.38531E-07	;		
RC(	735	) =	2.45582E-07	;		
RC(	736	) =	2.51933E-07	;		
RC(	737	) =	2.53849E-07	;		
RC(	738	) =	2.60635E-07	;		
RC(	739	) =	2.62915E-07	;		
RC(	740	) =	2.65429E-07	;		
RC(	741	) =	2.6968E-07	;		
RC(	742	) =	2.70991E-07	;		
RC(	743	) =	2.73545E-07	;		
RC(	744	) =	2.75262E-07	;		
RC(	745	) =	2.78355E-07	;		
RC(	746	) =	2.87642E-07	;		
RC(	747	) =	2.93782E-07	;		
RC(	748	) =	3.01524E-07	;		
RC(	749	) =	3.08341E-07	;		
RC(	750	) =	3.11406E-07	;		
RC(	751	) =	3.12454E-07	;		
RC(	752	) =	3.36524E-07	;		
RC(	753	) =	2.63271E-07	;		
RC(	754	) =	0	;		
RC(	755	) =	0	;		
RC(	756	) =	0	;		
RC(	757	) =	0	;		
RC(	758	) =	0	;		
RC(	759	) =	4.17167E-08	;		
RC(	760	) =	4.52435E-08	;		
RC(	761	) =	5.65148E-08	;		
RC(	762	) =	0	;		
RC(	763	) =	0	;		
RC(	764	) =	0	;		
RC(	765	) =	0	;		
RC(	766	) =	0	;		
RC(	767	) =	0	;		
RC(	768	) =	0	;		
RC(	769	) =	0	;		
RC(	770	) =	0	;		
RC(	771	) =	0	;		
RC(	772	) =	0	;		
RC(	773	) =	0	;		
RC(	774	) =	0	;		
RC(	775	) =	0	;		
RC(	776	) =	0	;		
RC(	777	) =	0	;		
RC(	778	) =	0	;		
RC(	779	) =	0	;		
RC(	780	) =	0	;		
RC(	781	) =	0	;		
RC(	782	) =	0	;		
RC(	783	) =	0	;		
RC(	784	) =	0	;		
RC(	785	) =	0	;		
RC(	786	) =	0	;		
RC(	787	) =	0	;		
RC(	788	) =	0	;		
RC(	789	) =	0	;		
RC(	790	) =	0	;		
RC(	791	) =	0	;		
RC(	792	) =	0	;		
RC(	793	) =	0	;		
RC(	794	) =	0	;		
RC(	795	) =	1.32584E-07	;		
RC(	796	) =	1.45795E-07	;		
RC(	797	) =	1.89045E-07	;		
RC(	798	) =	0	;		
RC(	799	) =	0	;		
RC(	800	) =	0	;		
RC(	801	) =	0	;		
RC(	802	) =	0	;		
RC(	803	) =	0	;		
RC(	804	) =	0	;		
RC(	805	) =	0	;		
RC(	806	) =	0	;		
RC(	807	) =	0	;		
RC(	808	) =	0	;		
RC(	809	) =	0	;		
RC(	810	) =	0	;		
RC(	811	) =	0	;		
RC(	812	) =	0	;		
RC(	813	) =	0	;		
RC(	814	) =	0	;		
RC(	815	) =	0	;		
RC(	816	) =	0	;		
RC(	817	) =	0	;		
RC(	818	) =	0	;		
RC(	819	) =	0	;		
RC(	820	) =	0	;		
RC(	821	) =	0	;		
RC(	822	) =	0	;		
RC(	823	) =	0	;		
RC(	824	) =	0	;		
RC(	825	) =	0	;		
RC(	826	) =	0	;		
RC(	827	) =	0	;		
RC(	828	) =	0	;		
RC(	829	) =	0	;		
RC(	830	) =	0	;		
RC(	831	) =	1.54549E-07	;		
RC(	832	) =	1.60781E-07	;		
RC(	833	) =	1.78075E-07	;		
RC(	834	) =	8.96496E-08	;		
RC(	835	) =	8.80942E-08	;		
RC(	836	) =	8.80831E-08	;		
RC(	837	) =	8.77208E-08	;		
RC(	838	) =	8.58392E-08	;		
RC(	839	) =	8.58281E-08	;		
RC(	840	) =	7.68145E-08	;		
RC(	841	) =	7.61648E-08	;		
RC(	842	) =	7.59024E-08	;		
RC(	843	) =	7.42056E-08	;		
RC(	844	) =	7.2677E-08	;		
RC(	845	) =	7.23054E-08	;		
RC(	846	) =	7.10707E-08	;		
RC(	847	) =	7.06558E-08	;		
RC(	848	) =	7.01983E-08	;		
RC(	849	) =	6.94248E-08	;		
RC(	850	) =	6.91863E-08	;		
RC(	851	) =	6.87215E-08	;		
RC(	852	) =	6.84091E-08	;		
RC(	853	) =	6.78463E-08	;		
RC(	854	) =	6.58251E-08	;		
RC(	855	) =	6.40377E-08	;		
RC(	856	) =	6.1804E-08	;		
RC(	857	) =	5.98752E-08	;		
RC(	858	) =	5.90083E-08	;		
RC(	859	) =	5.87126E-08	;		
RC(	860	) =	5.10613E-08	;		
RC(	861	) =	4.36438E-08	;		
RC(	862	) =	4.00672E-08	;		
RC(	863	) =	3.80691E-08	;		
RC(	864	) =	3.35166E-08	;		
RC(	865	) =	5.27361E-09	;		
RC(	866	) =	0	;		
RC(	867	) =	4.29729E-07	;		
RC(	868	) =	4.21603E-07	;		
RC(	869	) =	3.9563E-07	;		
RC(	870	) =	0	;		
RC(	871	) =	0	;		
RC(	872	) =	0	;		
RC(	873	) =	0	;		
RC(	874	) =	0	;		
RC(	875	) =	0	;		
RC(	876	) =	0	;		
RC(	877	) =	0	;		
RC(	878	) =	0	;		
RC(	879	) =	0	;		
RC(	880	) =	0	;		
RC(	881	) =	0	;		
RC(	882	) =	0	;		
RC(	883	) =	0	;		
RC(	884	) =	0	;		
RC(	885	) =	0	;		
RC(	886	) =	0	;		
RC(	887	) =	0	;		
RC(	888	) =	0	;		
RC(	889	) =	0	;		
RC(	890	) =	0	;		
RC(	891	) =	0	;		
RC(	892	) =	0	;		
RC(	893	) =	0	;		
RC(	894	) =	0	;		
RC(	895	) =	0	;		
RC(	896	) =	0	;		
RC(	897	) =	0	;		
RC(	898	) =	0	;		
RC(	899	) =	0	;		
RC(	900	) =	0	;		
RC(	901	) =	0	;		
RC(	902	) =	0	;		
RC(	903	) =	3.32608E-07	;		
RC(	904	) =	3.34784E-07	;		
RC(	905	) =	3.28676E-07	;		
RC(	906	) =	9.07362E-08	;		
RC(	907	) =	9.06548E-08	;		
RC(	908	) =	9.06542E-08	;		
RC(	909	) =	9.06352E-08	;		
RC(	910	) =	9.05367E-08	;		
RC(	911	) =	9.05361E-08	;		
RC(	912	) =	9.0064E-08	;		
RC(	913	) =	9.003E-08	;		
RC(	914	) =	9.00162E-08	;		
RC(	915	) =	8.99274E-08	;		
RC(	916	) =	8.98473E-08	;		
RC(	917	) =	8.98279E-08	;		
RC(	918	) =	8.97632E-08	;		
RC(	919	) =	8.97415E-08	;		
RC(	920	) =	8.97175E-08	;		
RC(	921	) =	8.9677E-08	;		
RC(	922	) =	8.96645E-08	;		
RC(	923	) =	8.96402E-08	;		
RC(	924	) =	8.96238E-08	;		
RC(	925	) =	8.95943E-08	;		
RC(	926	) =	8.94885E-08	;		
RC(	927	) =	8.93949E-08	;		
RC(	928	) =	8.92779E-08	;		
RC(	929	) =	8.91769E-08	;		
RC(	930	) =	8.91315E-08	;		
RC(	931	) =	8.9116E-08	;		
RC(	932	) =	8.87153E-08	;		
RC(	933	) =	8.83268E-08	;		
RC(	934	) =	8.81395E-08	;		
RC(	935	) =	8.80348E-08	;		
RC(	936	) =	8.77964E-08	;		
RC(	937	) =	8.63173E-08	;		
RC(	938	) =	8.15947E-08	;		
RC(	939	) =	0.000000627	;		
RC(	940	) =	0.000000636	;		
RC(	941	) =	0.000000696	;		
RC(	942	) =	0	;		
RC(	943	) =	0	;		
RC(	944	) =	0	;		
RC(	945	) =	0	;		
RC(	946	) =	0	;		
RC(	947	) =	0	;		
RC(	948	) =	0	;		
RC(	949	) =	0	;		
RC(	950	) =	0	;		
RC(	951	) =	0	;		
RC(	952	) =	0	;		
RC(	953	) =	0	;		
RC(	954	) =	0	;		
RC(	955	) =	0	;		
RC(	956	) =	0	;		
RC(	957	) =	0	;		
RC(	958	) =	0	;		
RC(	959	) =	0	;		
RC(	960	) =	0	;		
RC(	961	) =	0	;		
RC(	962	) =	0	;		
RC(	963	) =	0	;		
RC(	964	) =	0	;		
RC(	965	) =	0	;		
RC(	966	) =	0	;		
RC(	967	) =	0	;		
RC(	968	) =	0	;		
RC(	969	) =	0	;		
RC(	970	) =	0	;		
RC(	971	) =	0	;		
RC(	972	) =	0	;		
RC(	973	) =	0	;		
RC(	974	) =	0	;		
RC(	975	) =	0.000000708	;		
RC(	976	) =	0.000000756	;		
RC(	977	) =	0.000000693	;		
RC(	978	) =	0	;		
RC(	979	) =	0	;		
RC(	980	) =	0	;		
RC(	981	) =	0	;		
RC(	982	) =	0	;		
RC(	983	) =	0	;		
RC(	984	) =	0	;		
RC(	985	) =	0	;		
RC(	986	) =	0	;		
RC(	987	) =	0	;		
RC(	988	) =	0	;		
RC(	989	) =	0	;		
RC(	990	) =	0	;		
RC(	991	) =	0	;		
RC(	992	) =	0	;		
RC(	993	) =	0	;		
RC(	994	) =	0	;		
RC(	995	) =	0	;		
RC(	996	) =	0	;		
RC(	997	) =	0	;		
RC(	998	) =	0	;		
RC(	999	) =	0	;		
RC(	1000	) =	0	;		
RC(	1001	) =	0	;		
RC(	1002	) =	0	;		
RC(	1003	) =	0	;		
RC(	1004	) =	0	;		
RC(	1005	) =	0	;		
RC(	1006	) =	0	;		
RC(	1007	) =	0	;		
RC(	1008	) =	0	;		
RC(	1009	) =	0	;		
RC(	1010	) =	0	;		
RC(	1011	) =	2.23138E-07	;		
RC(	1012	) =	2.21539E-07	;		
RC(	1013	) =	2.16429E-07	;		
RC(	1014	) =	9.58867E-08	;		
RC(	1015	) =	7.77065E-08	;		
RC(	1016	) =	7.75768E-08	;		
RC(	1017	) =	7.33424E-08	;		
RC(	1018	) =	5.7707E-08	;		
RC(	1019	) =	5.75935E-08	;		
RC(	1020	) =	4.54991E-10	;		
RC(	1021	) =	6.76933E-10	;		
RC(	1022	) =	1.21856E-09	;		
RC(	1023	) =	1.62993E-09	;		
RC(	1024	) =	8.61145E-09	;		
RC(	1025	) =	2.95115E-09	;		
RC(	1026	) =	4.4626E-09	;		
RC(	1027	) =	1.19628E-09	;		
RC(	1028	) =	3.41603E-09	;		
RC(	1029	) =	2.37426E-09	;		
RC(	1030	) =	8.94598E-10	;		
RC(	1031	) =	4.72696E-11	;		
RC(	1032	) =	0	;		
RC(	1033	) =	0	;		
RC(	1034	) =	0	;		
RC(	1035	) =	0	;		
RC(	1036	) =	0	;		
RC(	1037	) =	0	;		
RC(	1038	) =	0	;		
RC(	1039	) =	0	;		
RC(	1040	) =	0	;		
RC(	1041	) =	0	;		
RC(	1042	) =	0	;		
RC(	1043	) =	0	;		
RC(	1044	) =	0	;		
RC(	1045	) =	0	;		
RC(	1046	) =	0	;		
RC(	1047	) =	2.06492E-06	;		
RC(	1048	) =	2.04586E-06	;		
RC(	1049	) =	1.98493E-06	;		
RC(	1050	) =	5.47535E-07	;		
RC(	1051	) =	3.30747E-07	;		
RC(	1052	) =	3.29201E-07	;		
RC(	1053	) =	2.78708E-07	;		
RC(	1054	) =	2.32245E-08	;		
RC(	1055	) =	2.24635E-08	;		
RC(	1056	) =	2.27795E-11	;		
RC(	1057	) =	8.0192E-12	;		
RC(	1058	) =	5.2198E-12	;		
RC(	1059	) =	1.85693E-13	;		
RC(	1060	) =	7.7992E-15	;		
RC(	1061	) =	0	;		
RC(	1062	) =	0	;		
RC(	1063	) =	0	;		
RC(	1064	) =	0	;		
RC(	1065	) =	0	;		
RC(	1066	) =	0	;		
RC(	1067	) =	0	;		
RC(	1068	) =	0	;		
RC(	1069	) =	0	;		
RC(	1070	) =	0	;		
RC(	1071	) =	0	;		
RC(	1072	) =	0	;		
RC(	1073	) =	0	;		
RC(	1074	) =	0	;		
RC(	1075	) =	0	;		
RC(	1076	) =	0	;		
RC(	1077	) =	0	;		
RC(	1078	) =	0	;		
RC(	1079	) =	0	;		
RC(	1080	) =	0	;		
RC(	1081	) =	0	;		
RC(	1082	) =	0	;		
RC(	1083	) =	5.07642E-08	;		
RC(	1084	) =	5.03897E-08	;		
RC(	1085	) =	4.91929E-08	;		
RC(	1086	) =	2.09576E-08	;		
RC(	1087	) =	1.66992E-08	;		
RC(	1088	) =	1.66688E-08	;		
RC(	1089	) =	1.56769E-08	;		
RC(	1090	) =	1.023E-08	;		
RC(	1091	) =	1.01962E-08	;		
RC(	1092	) =	1.39454E-10	;		
RC(	1093	) =	1.0363E-10	;		
RC(	1094	) =	8.91577E-11	;		
RC(	1095	) =	2.89437E-11	;		
RC(	1096	) =	5.49825E-12	;		
RC(	1097	) =	0	;		
RC(	1098	) =	0	;		
RC(	1099	) =	0	;		
RC(	1100	) =	0	;		
RC(	1101	) =	0	;		
RC(	1102	) =	0	;		
RC(	1103	) =	0	;		
RC(	1104	) =	0	;		
RC(	1105	) =	0	;		
RC(	1106	) =	0	;		
RC(	1107	) =	0	;		
RC(	1108	) =	0	;		
RC(	1109	) =	0	;		
RC(	1110	) =	0	;		
RC(	1111	) =	0	;		
RC(	1112	) =	0	;		
RC(	1113	) =	0	;		
RC(	1114	) =	0	;		
RC(	1115	) =	0	;		
RC(	1116	) =	0	;		
RC(	1117	) =	0	;		
RC(	1118	) =	0	;		
RC(	1119	) =	2.99627E-08	;		
RC(	1120	) =	2.9796E-08	;		
RC(	1121	) =	2.9263E-08	;		
RC(	1122	) =	1.66891E-08	;		
RC(	1123	) =	1.47926E-08	;		
RC(	1124	) =	1.47791E-08	;		
RC(	1125	) =	1.43374E-08	;		
RC(	1126	) =	1.36484E-08	;		
RC(	1127	) =	1.36982E-08	;		
RC(	1128	) =	4.74878E-09	;		
RC(	1129	) =	5.93451E-09	;		
RC(	1130	) =	6.20528E-09	;		
RC(	1131	) =	9.90583E-09	;		
RC(	1132	) =	1.37277E-08	;		
RC(	1133	) =	1.34078E-08	;		
RC(	1134	) =	1.64922E-08	;		
RC(	1135	) =	1.62929E-08	;		
RC(	1136	) =	1.69703E-08	;		
RC(	1137	) =	1.78288E-08	;		
RC(	1138	) =	1.87436E-08	;		
RC(	1139	) =	1.88192E-08	;		
RC(	1140	) =	1.93247E-08	;		
RC(	1141	) =	2.00124E-08	;		
RC(	1142	) =	1.81208E-08	;		
RC(	1143	) =	1.55825E-08	;		
RC(	1144	) =	1.25264E-08	;		
RC(	1145	) =	7.26037E-09	;		
RC(	1146	) =	6.92708E-09	;		
RC(	1147	) =	9.02317E-09	;		
RC(	1148	) =	1.25064E-09	;		
RC(	1149	) =	7.61317E-09	;		
RC(	1150	) =	1.10986E-08	;		
RC(	1151	) =	1.30457E-08	;		
RC(	1152	) =	1.74821E-08	;		
RC(	1153	) =	4.5005E-08	;		
RC(	1154	) =	1.32878E-07	;		
RC(	1155	) =	0	;		
RC(	1156	) =	0	;		
RC(	1157	) =	0	;		
RC(	1158	) =	0	;		
RC(	1159	) =	0	;		
RC(	1160	) =	0	;		
RC(	1161	) =	0	;		
RC(	1162	) =	0	;		
RC(	1163	) =	0	;		
RC(	1164	) =	0	;		
RC(	1165	) =	0	;		
RC(	1166	) =	0	;		
RC(	1167	) =	0	;		
RC(	1168	) =	0	;		
RC(	1169	) =	0	;		
RC(	1170	) =	0	;		
RC(	1171	) =	0	;		
RC(	1172	) =	0	;		
RC(	1173	) =	0	;		
RC(	1174	) =	0	;		
RC(	1175	) =	0	;		
RC(	1176	) =	0	;		
RC(	1177	) =	6.16136E-10	;		
RC(	1178	) =	3.89204E-09	;		
RC(	1179	) =	7.68057E-09	;		
RC(	1180	) =	1.78903E-08	;		
RC(	1181	) =	2.95126E-08	;		
RC(	1182	) =	3.32084E-08	;		
RC(	1183	) =	3.58901E-08	;		
RC(	1184	) =	1.21918E-07	;		
RC(	1185	) =	0	;		
RC(	1186	) =	0	;		
RC(	1187	) =	0	;		
RC(	1188	) =	0	;		
RC(	1189	) =	0	;		
RC(	1190	) =	0	;		
RC(	1191	) =	0	;		
RC(	1192	) =	0	;		
RC(	1193	) =	0	;		
RC(	1194	) =	0	;		
RC(	1195	) =	0	;		
RC(	1196	) =	0	;		
RC(	1197	) =	6.41704E-10	;		
RC(	1198	) =	1.60622E-08	;		
RC(	1199	) =	1.62658E-08	;		
RC(	1200	) =	1.711E-08	;		
RC(	1201	) =	5.058E-09	;		
RC(	1202	) =	2.13972E-09	;		
RC(	1203	) =	2.43209E-10	;		
RC(	1204	) =	1.35527E-11	;		
RC(	1205	) =	1.09169E-11	;		
RC(	1206	) =	9.96711E-13	;		
RC(	1207	) =	6.28589E-13	;		
RC(	1208	) =	4.36717E-13	;		
RC(	1209	) =	3.66104E-13	;		
RC(	1210	) =	3.56632E-13	;		
RC(	1211	) =	3.76806E-13	;		
RC(	1212	) =	4.00613E-13	;		
RC(	1213	) =	4.9653E-13	;		
RC(	1214	) =	1.30172E-12	;		
RC(	1215	) =	3.56591E-12	;		
RC(	1216	) =	1.24375E-11	;		
RC(	1217	) =	2.59959E-11	;		
RC(	1218	) =	2.56364E-11	;		
RC(	1219	) =	2.69812E-11	;		
RC(	1220	) =	1.43762E-10	;		
RC(	1221	) =	1.68682E-10	;		
RC(	1222	) =	1.86917E-10	;		
RC(	1223	) =	1.97104E-10	;		
RC(	1224	) =	2.20315E-10	;		
RC(	1225	) =	3.6431E-10	;		
RC(	1226	) =	8.24047E-10	;		
end
